local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

local AceEvent = LibStub:GetLibrary("AceEvent-3.0")

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- the Algorithms module
local Algorithms = Addon:NewModule("Algorithms", "AceEvent-3.0")

-- local functions
local tinsert           = table.insert
local pairs             = pairs

-- algorithm
NS.ALGORITHM  = "algorithm"

local Algorithm_Prototype = {}

-- algorithm functions
function Algorithm_Prototype:CalculateForce(inlet, pX, pY, zoneName, zoneMap)
	Addon:Debug(self:GetName() .. " CalculateForce: empty prototype")
	if not self:IsReady() then
		return
	end
	
	-- empty dummy, overwrite in specific implementation
end

-- options table
function Algorithm_Prototype:SetupOptions(db)
	Addon:Debug(self:GetName() .. " SetupOptions: empty prototype")
	if type(db) ~= "table" then		
		return {}
	end
	
	-- empty dummy, overwrite in specific implementation
end

-- factory method for algorithm creation
function NS:CreateAlgorithm(name, description)
	local module = NS:CreateModule(NS.ALGORITHM, name, description)
	
	module.nodes = {}
	
	for k, v in pairs(Algorithm_Prototype) do
		module[k] = v
	end
	
	AceEvent:Embed(module)
	
	return module
end

function NS:IsValidAlgorithm(module)
	if not NS:IsValidModule(module) or module:GetType() ~= NS.ALGORITHM then
		return false
	end

	for k, _ in pairs(Algorithm_Prototype) do
		if not module[k] then
			return false
		end
	end
	
	return true
end

Algorithms.NONE = "none"

-- module data
local moduleData = {
	algorithms   = {},
	activeName   = "none",
}

-- algorithm handling
function Algorithms:OnInitialize()
	-- empty
end

function Algorithms:OnEnable()
	self:RegisterAlgorithms()
end

function Algorithms:OnDisable()
	self:UnregisterAlgorithms()
end

function Algorithms:RegisterAlgorithms()
	for name, algorithm in NS:IterateModulesByType(NS.ALGORITHM) do
		self:RegisterAlgorithm(algorithm)
	end
end

function Algorithms:RegisterAlgorithm(algorithm)
	if not algorithm then 
		self:Debug("RegisterAlgorithm: no algorithm provided")
		return false
	end
	
	-- check interface
	if not NS:IsValidAlgorithm(algorithm)	then
		self:Debug("RegisterAlgorithm: algorithm doesn't implement interface")
		return false
	end
	
	local name = algorithm:GetName()
	
	if not name or name == "" then
		self:Debug("RegisterAlgorithm: algorithm doesn't provide name")
		return false
	end

	if name == self.NONE then
		self:Debug("RegisterAlgorithm: algorithm provides reserved name '" .. self.NONE .. "'")
		return false
	end
	
	-- already registered
	if moduleData.algorithms[name] then
		self:Debug("RegisterAlgorithm: algorithm '" .. tostring(name) .. "' already registered")
		return false
	end
	
	-- register algorithm
	moduleData.algorithms[name] = algorithm
	
	self:SendMessage("MAGNETO_ALGORITHM_REGISTERED_CHANGED", name, algorithm, true)
	
	return true
end

function Algorithms:UnregisterAlgorithms()
	for name, algorithm in pairs(moduleData.algorithms) do
		self:UnregisterAlgorithm(algorithm)
	end
end

function Algorithms:UnregisterAlgorithm(algorithm)
	if not algorithm or not algorithm.GetName or not self:HasAlgorithm(algorithm:GetName()) then 
		return
	end
	
	local name = algorithm:GetName()
	
	-- deactivate
	if name == self:GetActiveAlgorithmName() then
		self:SetActiveAlgorithm(self.NONE)
	end
	
	-- unregister algorithm
	moduleData.algorithms[name] = nil
	
	self:SendMessage("MAGNETO_ALGORITHM_REGISTERED_CHANGED", name, algorithm, false)	
end

function Algorithms:HasAlgorithm(name)
	return moduleData.algorithms[name] and true or false
end

function Algorithms:GetAlgorithm(name)
	return moduleData.algorithms[name]
end

function Algorithms:IsAlgorithmOperational()
	local algorithm = self:GetActiveAlgorithm()
	
	return algorithm and algorithm:IsReady() or false
end

function Algorithms:IsAlgorithmReady(name)
	return self:HasAlgorithm(name) and moduleData.algorithms[name]:IsReady() or false
end

function Algorithms:IsAlgorithmActive(name)
	return self:GetActiveAlgorithmName() == name
end

function Algorithms:SetActiveAlgorithm(name)
	if not self:HasAlgorithm(name) and name ~= self.NONE then
		return
	end

	if moduleData.activeName == name then
		return
	end
	
	moduleData.activeName = name
	
	self:SendMessage("MAGNETO_ALGORITHM_ACTIVE_CHANGED", name)
end

function Algorithms:GetActiveAlgorithmName()
	return moduleData.activeName
end

function Algorithms:GetActiveAlgorithm()
	return self:GetAlgorithm(moduleData.activeName)
end

function Algorithms:IterateAlgorithms()
	return pairs(moduleData.algorithms)
end

-- test
function Algorithms:Debug(msg)
	Addon:Debug("(Algorithms) " .. msg)
end
