local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- setup libs
local LibStub   = LibStub
local LDB       = LibStub:GetLibrary("LibDataBroker-1.1")
local AceTimer  = LibStub:GetLibrary("AceTimer-3.0")

-- the module name
local MODULE      = "Minimap Button"
local DESCRIPTION = L["Module that provides direction arrow and force representation as Minimap-Button."]

-- local functions
local cos     = math.cos
local sin     = math.sin

local GetCVar         = _G.GetCVar
local GetPlayerFacing = _G.GetPlayerFacing

local PI               = math.pi
local ONE_DIV_PI       = 1 / PI

local ARROW = "Interface\\Addons\\"..ADDON.."\\arrow.tga"

-- aux variables
local _

-- the module
local Outlet = NS:CreateOutlet(MODULE, DESCRIPTION)

AceTimer:Embed(Outlet)

Outlet.magnitude = 0
Outlet.direction = 0
Outlet.force     = 0
Outlet.angle     = 0

-- minimap button handling
Outlet.show      = false
Outlet.button    = nil
Outlet.dragMode  = nil

local function MoveButton(self)
	if MinimapButton.dragMode == "free" then
		local centerX, centerY = Minimap:GetCenter()
		local x, y = GetCursorPosition()
		x, y = x / self:GetEffectiveScale() - centerX, y / self:GetEffectiveScale() - centerY
		self:ClearAllPoints()
		self:SetPoint("CENTER", x, y)
	else
		local centerX, centerY = Minimap:GetCenter()
		local x, y = GetCursorPosition()
		x, y = x / self:GetEffectiveScale() - centerX, y / self:GetEffectiveScale() - centerY
		centerX, centerY = abs(x), abs(y)
		centerX, centerY = (centerX / sqrt(centerX^2 + centerY^2)) * 80, (centerY / sqrt(centerX^2 + centerY^2)) * 80
		centerX = x < 0 and -centerX or centerX
		centerY = y < 0 and -centerY or centerY
		self:ClearAllPoints()
		self:SetPoint("CENTER", centerX, centerY)
	end
end

function Outlet:CreateMinimapButton()
	local button = CreateFrame("Button", ADDON .. "_" .. MODULE .. "_MinimapButton", Minimap)
	button:SetHeight(32)
	button:SetWidth(32)
	button:SetFrameStrata("HIGH")
	button:SetPoint("CENTER", -77.44, -20.06)
	button:SetMovable(true)
	button:SetUserPlaced(true)
	button:EnableMouse(true)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	
	local bg = button:CreateTexture(button:GetName() .. "BG", "HIGH")
	bg:SetTexture(0, 0, 0)
	bg:SetWidth(22)
	bg:SetHeight(22)
	bg:SetPoint("CENTER", button, "CENTER", 0, 0)
	
	button.bg = bg
	
	local icon = button:CreateTexture(button:GetName() .. "Icon", "HIGH")
	icon:SetTexture(ARROW)
	icon:SetTexCoord(0, 1, 0, 1)
	icon:SetWidth(22)
	icon:SetHeight(22)
	icon:SetPoint("CENTER", button, "CENTER", 0, 0)
	
	button.icon = icon
	
	local overlay = button:CreateTexture(button:GetName() .. "Overlay", "OVERLAY")
	overlay:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
	overlay:SetWidth(53)
	overlay:SetHeight(53)
	overlay:SetPoint("TOPLEFT", button, "TOPLEFT")
	
	button:SetScript("OnClick", function(self, mousebutton)
		Addon:OnClick(mousebutton)
	end)
	
	button:SetScript("OnMouseDown", function(self, mousebutton)
		if IsShiftKeyDown() and IsControlKeyDown() then
			Outlet.dragMode = "free"
			self:SetScript("OnUpdate", MoveButton)
		elseif IsShiftKeyDown() then
			Outlet.dragMode = nil
			self:SetScript("OnUpdate", MoveButton)
		end
	end)
	
	button:SetScript("OnMouseUp", function(self)
		self:SetScript("OnUpdate", nil)
	end)

	button:SetScript("OnEnter", function(self)
		Addon:CreateTooltip(self)
	end)
	
	button:SetScript("OnLeave", function(self)
		Addon:RemoveTooltip()
	end)
	
	button:Hide()
	
	self.button = button
end

function Outlet:SetShow(show)
	show = show and true or false

	if show ~= self.show then
		self.show = show
		
		if Outlet.show then
			Outlet.button:Show()
		else
			Outlet.button:Hide()
		end
	end
end

function Outlet:GetShow()
	return self.show
end

Outlet:CreateMinimapButton()

Outlet:SetReady(true)

-- register
NS:RegisterModule(Outlet)

-- implement functions for output
function Outlet:HandleActiveChanged()
	if not self:IsReady() then
		return
	end
	
	if self:IsActive() then
		-- setup timer if not present
		if not self.timer then
			self:ScheduleRepeatingTimer("UpdateIcon", 0.1)
		end
	else
		-- reset timer
		if self.timer then
			self:CancelTimer(self.timer)
		end
	end
	
	self:SetShow(self:IsActive())
end

function Outlet:Refresh()
	if not self:IsOperational() then
		return
	end
	
	local magnitude, direction = Addon:GetForce()

	if magnitude == 0 then
		self:Clear()
		return
	end
	
	self.magnitude    = magnitude
	self.direction    = direction	
end

function Outlet:Clear()
	if not self:IsReady() then
		return
	end

	self.magnitude = 0
	self.direction = 0	
	
	-- point upward
	self:SetIconTexCoord(0, 0, 0, 1, 1, 0, 1, 1)
	
	-- gray out arrow
	self:SetIconColor(0.6, 0.6, 0.6)
end

function Outlet:UpdateIcon()
	if not self:IsOperational() or self.magnitude == 0 then
		return
	end

	local facing = - self.direction + GetPlayerFacing()
		
	local xUL, yUL, xLL, yLL, xUR, yUR, xLR, yLR
	
	local cosDir = cos(facing)
	local sinDir = sin(facing)
	
	local gX = (sinDir + cosDir) * 0.5
	local gY = (sinDir - cosDir) * 0.5
	
	xUL = 0.5 - gX
	yUL = 0.5 + gY
	xLL = 0.5 + gY
	yLL = 0.5 + gX
	xUR = 0.5 - gY
	yUR = 0.5 - gX
	xLR = 0.5 + gX
	yLR = 0.5 - gY
	
	self:SetIconTexCoord(xUL, yUL, xLL, yLL, xUR, yUR, xLR, yLR)
	self:SetDirectionColor(facing)
end

function Outlet:SetIconTexCoord(xUL, yUL, xLL, yLL, xUR, yUR, xLR, yLR)
	if self.button and self.button.icon then
		self.button.icon:SetTexCoord(xUL, yUL, xLL, yLL, xUR, yUR, xLR, yLR)
	end
end

function Outlet:GetIconTexCoord()
	if self.button and self.button.icon then
		self.button.icon:GetTexCoord()
	end
end

-- can colors be done?
function Outlet:SetDirectionColor(angle)
	local perc = abs(PI - abs(angle)) * ONE_DIV_PI
    local g = 1
    local r = 0
	
    if perc > 0.5 then
      g = 1
      r = perc * -2 + 2
    else
      r = 1
      g = perc * 2
    end
	
    self:SetIconColor(r, g, 0)
end

function Outlet:SetIconColor(r, g, b)
	if self.button and self.button.icon then
		self.button.icon:SetVertexColor(r, g, b)
	end
end

-- test
function Outlet:Debug(msg)
	Addon:Debug("(Outlet:" .. MODULE .. ") " .. msg)
end
