-- module to notify addon of zone changes
-- used instead of MapChanged event of LibMapData because of 2 reasons:
-- 1) LibMapData will not fire zone changed events when change the zone with open world map and then close the map
-- 2) when entering the garrison the player coords are shortly (0, 0) which causes LibMapData to ignore the change
local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- the ZoneTracker module
local ZoneTracker = Addon:NewModule("ZoneTracker", "AceEvent-3.0")

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- LibMapData
local LibMapData = LibStub:GetLibrary("LibMapData-1.0")

-- local functions
local ipairs   = ipairs 
local pairs    = pairs 
local strlower = strlower
local strfind  = strfind
local tinsert  = table.insert
local tsort    = table.sort

local GetCurrentMapDungeonLevel = GetCurrentMapDungeonLevel
local GetMapInfo                = GetMapInfo
local GetPlayerMapPosition      = GetPlayerMapPosition
local SetMapToCurrentZone       = SetMapToCurrentZone

-- aux variables
local _

-- local variables
local lastMap = nil
local lastFloor = 0
local mapChanged = false

local watchFrame = CreateFrame("Frame")

-- module handling
function ZoneTracker:OnInitialize()
	-- empty
end

function ZoneTracker:OnEnable()
	self:SetupEvents()

	self:UpdateZone(true)
end

function ZoneTracker:OnDisable()
	self:ShutdownEvents()
end

function ZoneTracker:SetupEvents()
	-- zone changes
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "CheckZone")
	self:RegisterEvent("ZONE_CHANGED", "CheckZone")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "CheckZone")
	self:RegisterEvent("ZONE_CHANGED_INDOORS", "CheckZone")

	-- world map changes
	self:RegisterEvent("WORLD_MAP_UPDATE", "CheckWorldMap")	
	
	watchFrame:SetScript("OnUpdate", function(self, elapsed) ZoneTracker:UpdateWorldMap() end)
end

function ZoneTracker:ShutdownEvents()
	-- zone changes
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self:UnregisterEvent("ZONE_CHANGED")
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
	self:UnregisterEvent("ZONE_CHANGED_INDOORS")

	-- world map changes
	self:UnregisterEvent("WORLD_MAP_UPDATE")
	
	watchFrame:SetScript("OnUpdate", nil)	
end

function ZoneTracker:CheckWorldMap()
	if WorldMapFrame:IsVisible() then
		mapChanged = true;
	end
end

function ZoneTracker:UpdateWorldMap()
	if mapChanged and not WorldMapFrame:IsVisible() then
		self:UpdateZone(false)
		
		mapChanged = false;
	end
end

function ZoneTracker:CheckZone()
	self:UpdateZone(false)
end

function ZoneTracker:UpdateZone(force)
	if WorldMapFrame:IsVisible() then 
		return 
	end
	
	SetMapToCurrentZone()
	
	local x,y = GetPlayerMapPosition("player")

	local map = GetMapInfo()
		
	-- check if map exists in LibMapData
	if x == 0 and y == 0 and not LibMapData:MapAreaId(map) then
		return
	end
	
	if map == nil then
		return
	end
	
	local floor = GetCurrentMapDungeonLevel()
	
	if map ~= lastMap or floor ~= lastFloor or force then
		lastMap = map
		lastFloor = floor
		
		Addon:UpdateZone("MapChanged", map, floor)
	end
end

-- test
function ZoneTracker:Debug(msg)
	Addon:Debug("(ZoneTracker) " .. msg)
end
