﻿--[[
Copyright (c) 2014, Bernhard Voigt <tritium@noisywires.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the copyright owner nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "LibNodeTypeInfo-1.0"
local MINOR_VERSION = 1

if not LibStub then error(MAJOR_VERSION .. " requires LibStub") end

local NTI, oldLib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)

if not NTI then
	return
end

if oldLib then
	oldLib = {}
	for k, v in pairs(NTI) do
		NTI[k] = nil
		oldLib[k] = v
	end
end

local GetMapNameByID    = _G.GetMapNameByID
local GetProfessions    = _G.GetProfessions
local GetProfessionInfo = _G.GetProfessionInfo

local LBHN = LibStub("LibBabble-HerbNodes-1.0")
local LBMN = LibStub("LibBabble-MineralNodes-1.0")
local LBFN = LibStub("LibBabble-FishingNodes-1.0")
local LBLN = LibStub("LibBabble-LoggingNodes-1.0")

local BHN = LBHN and LBHN:GetLookupTable()
local BMN = LBMN and LBMN:GetLookupTable()
local BFN = LBFN and LBFN:GetLookupTable()
local BLN = LBLN and LBLN:GetLookupTable()

-- this matches the skillLine return value of GetProfessionInfo()
NTI.PROFESSION_MINING    =    186 -- skill id
NTI.PROFESSION_HERBALISM =    182 -- skill id
NTI.PROFESSION_FISHING   =    356 -- skill id
NTI.PROFESSION_LOGGING   = 167924 -- since harvesting lumber requires no profession we use the tracking spell id instead

local SKILL_MAX_BASE     =    300

-- NOTE: complete the following series logically
--       75, 150, 225, 300, 375, 450, 525, 600, ???
--       Well done, Blizzard!
local profiencyLevelRange = {
	[0] = 75,
	[8] = 100,
}

local professionDB = {
	["mining"]    = NTI.PROFESSION_MINING,
	["herbalism"] = NTI.PROFESSION_HERBALISM,
	["fishing"]   = NTI.PROFESSION_FISHING,
	["logging"]   = NTI.PROFESSION_LOGGING,
}

local professionNameDB = {
}

local nodeDB = {
	[NTI.PROFESSION_MINING]    = nil,
	[NTI.PROFESSION_HERBALISM] = nil,
	[NTI.PROFESSION_FISHING]   = nil,
	[NTI.PROFESSION_LOGGING]   = nil,
}

local lootDB = {
	-- material id -> node id list
	[NTI.PROFESSION_MINING]    = nil,
	[NTI.PROFESSION_HERBALISM] = nil,
	[NTI.PROFESSION_FISHING]   = nil,
	[NTI.PROFESSION_LOGGING]   = nil,
}

local zoneDB = {
	-- zone id -> node id list
	[NTI.PROFESSION_MINING]    = nil,
	[NTI.PROFESSION_HERBALISM] = nil,
	[NTI.PROFESSION_FISHING]   = nil,
	[NTI.PROFESSION_LOGGING]   = nil,
}

local spellDB = {
	-- skill -> spell
	[NTI.PROFESSION_MINING]    = {
		 [75] =   2575, -- apprentice
		[150] =   2576, -- journeyman
		[225] =   3564, -- expert
		[300] =  10248, -- artisan
		[375] =  29354, -- master
		[450] =  50310, -- grand master
		[525] =  74517, -- illustrous grand master
		[600] = 102161, -- zen master
		[700] = 158754, -- draenor master
	},
	[NTI.PROFESSION_HERBALISM] = {
		 [75] =   2366, -- apprentice
		[150] =   2368, -- journeyman
		[225] =   3570, -- expert
		[300] =  11993, -- artisan
		[375] =  28695, -- master
		[450] =  50300, -- grand master
		[525] =  74519, -- illustrous grand master
		[700] = 158745, -- draenor master
	},
	[NTI.PROFESSION_FISHING] = {
		 [75] =   7620, -- apprentice
		[150] =   7731, -- journeyman
		[225] =   7732, -- expert
		[300] =  18248, -- artisan
		[375] =  33095, -- master
		[450] =  51294, -- grand master
		[525] =  88868, -- illustrous grand master
		[700] = 158743, -- draenor master
	},
	[NTI.PROFESSION_LOGGING] = {
		 [75] = 167924, -- since harvesting lumber requires no profession we use the tracking spell id instead
		[150] = 167924, -- 
		[225] = 167924, -- 
		[300] = 167924, -- 
		[375] = 167924, -- 
		[450] = 167924, -- 
		[525] = 167924, -- 
		[600] = 167924, -- 
		[700] = 167924, -- 167895, 167896, 167897
	},
}

local trackingDB = {
	-- tracking spell for profession
	[NTI.PROFESSION_MINING]    = 2580,
	[NTI.PROFESSION_HERBALISM] = 2383,
	[NTI.PROFESSION_FISHING]   = 43308,
	[NTI.PROFESSION_LOGGING]   = 167924,
}

-- translate mining node
local function TMN(name)
	if not name then
		return ""
	end
	
	return BMN and BMN[name] or name
end 

-- translate herb node
local function THN(name)
	if not name then
		return ""
	end
	
	return BHN and BHN[name] or name
end 

-- translate fishing node
local function TFN(name)
	if not name then
		return ""
	end
	
	return BFN and BFN[name] or name
end 

-- translate lumber node
local function TLN(name)
	if not name then
		return ""
	end
	
	return BLN and BLN[name] or name
end 

-- returns profession spell for profession name
function NTI:GetProfessions()
	local professions = {}
	
	for name, id in pairs(professionDB) do
		profession[name] = id
	end

	return professions
end

-- returns profession spell for profession name
function NTI:GetProfessionByLibName(professionName)
	return professionDB[professionName]
end

-- returns localized name for profession
function NTI:GetProfessionName(profession)
	if not nodeDB[profession] then
		return
	end

	if not professionNameDB[profession] then
		if profession == NTI.PROFESSION_LOGGING then
			professionNameDB[profession] = GetSpellInfo(NTI.PROFESSION_LOGGING)
		else
			local profIndexes = { table.pack(GetProfessions()) }
			
			for _, profIdx in pairs(profIndexes) do
				local name, _, _, _, _, _, skillLine = GetProfessionInfo(index)
				
				if skillLine == profession then
					professionNameDB[profession] = name
				end
			end
		end
	end

	return professionNameDB[profession]
end

-- returns tracking spell for profession
function NTI:GetTrackingSpell(profession)
	return trackingDB[profession]
end

-- returns expansion id for skill
function NTI:GetExpansionForSkill(skill)
	if type(skill) ~= "number" then
		return nil
	end

	if skill < 0 then
		skill = 0
	end
	
	local level = 0
	local maxSkill = 0
	local range = 0
	local expansion = 0
	
	while maxSkill < skill do
		if profiencyLevelRange[level] then
			range = profiencyLevelRange[level]
		end
		
		maxSkill = maxSkill + range
		level = level + 1
		
		if maxSkill > SKILL_MAX_BASE then
			expansion = expansion + 1
		end
	end
	
	return expansion
end

-- returns expansion id for skill
function NTI:GetMaxSkillForExpansion(expansion)
	if type(expansion) ~= "number" then
		return nil
	end
	
	if expansion < 0 then
		expansion = 0
	end
	
	local maxSkillExpansion = SKILL_MAX_BASE	
	local maxSkill = 0
	local range = 0
	
	for level = 0, expansion, 1 do
		if profiencyLevelRange[level] then
			range = profiencyLevelRange[level]
		end

		maxSkill = maxSkill + range
		
		if maxSkill > SKILL_MAX_BASE then
			maxSkillExpansion = maxSkill
		end		
	end
	
	return maxSkillExpansion
end

-- returns the proficiency level for given skill
--		[0] =  75, -- apprentice
--		[1] = 150, -- journeyman
--		[2] = 225, -- expert
--		[3] = 300, -- artisan
--		[4] = 375, -- master
--		[5] = 450, -- grand master
--		[6] = 525, -- illustrous grand master
--		[7] = 600, -- zen master
--		[8] = 700, -- draenor master
function NTI:GetProficiencyLevelForSkill(skill)
	if type(skill) ~= "number" then
		return nil
	end

	if skill < 0 then
		skill = 0
	end
	
	local level = 0
	local maxSkill = 0
	local range = 0
	local expansion = 0
	
	while maxSkill < skill do
		if profiencyLevelRange[level] then
			range = profiencyLevelRange[level]
		end
		
		maxSkill = maxSkill + range
		level = level + 1
	end
	
	return level
end

-- returns profession spell id for gathering skill
function NTI:GetRequiredSpellForSkill(profession, skill)
	if type(skill) ~= "number" then
		return nil
	end
		
	local spell = nil

	local spellData = spellDB[profession]
	
	if skill and spellData then
		spell = spellData[self:GetProficiencyLevelForSkill(skill)]
	end
	
	return spell
end

-- returns profession for given id
function NTI:GetProfession(id)
	if nodeDB[self.PROFESSION_MINING][id] then
		return self.PROFESSION_MINING
	elseif nodeDB[self.PROFESSION_HERBALISM][id] then
		return self.PROFESSION_HERBALISM
	elseif nodeDB[self.PROFESSION_FISHING][id] then
		return self.PROFESSION_FISHING
	elseif nodeDB[self.PROFESSION_LOGGING][id] then
		return self.PROFESSION_LOGGING
	end
end

-- returns if a node type exists
function NTI:Exists(profession, id)
	return nodeDB[profession] and nodeDB[profession][id] and true or false
end

-- returns non-localized english name
function NTI:GetName(profession, id)
	return nodeDB[profession] and nodeDB[profession][id] and nodeDB[profession][id].name
end

-- returns id for localized name
function NTI:GetID(profession, name)
	if not nodeDB[profession] then
		return
	end

	for id, data in pairs(nodeDB[profession]) do
		if data.name == name then
			return id
		end
	end
	
	return
end

-- returns copy of node data
function NTI:GetInfoData(profession, id)
	local info = nil

	local data = nodeDB[profession] and nodeDB[profession][id]

	if data then
		info = {}
		
		info.id   = data.id
		info.name = data.name
		info.profession = data.profession
		
		info.contains = {}

		for id, item in pairs(data.contains) do
			info.contains[id] = {
				rate      = item.rate,
				minAmount = item.minAmount or 1,
				maxAmount = item.maxAmount or 1,
			}
		end
		
		info.mainLoot = data.mainLoot
		
		info.zones = {}
		
		for zone, id in pairs(data.zones) do
			info.zones[zone] = id
		end
		
		info.quest = data.quest
				
		info.expansion = data.expansion
	end
	
	return info
end

-- returns id of expansion for a given node
function NTI:GetExpansion(profession, id)
	return nodeDB[profession] and nodeDB[profession][id] and nodeDB[profession][id].expansion
end

-- Iterators:
--
-- list of node ids that fulfill certain criteria
-- NTI:IterateNodes(profession)
-- NTI:IterateNodesForExpansion(profession, expansion)
-- NTI:IterateNodesForZone(profession, zone)
-- NTI:IterateNodesForContinent(profession, continent)
-- NTI:IterateNodesForLoot(profession, item)
--
-- list of properties of a node
-- NTI:IterateZonesForNode(profession, id) -- returns zone name and zone id
-- NTI:IterateLootForNode(profession, id)
do --Do-end block for iterators
	local emptyTbl = {}
	local tablestack = setmetatable({}, {__mode = 'k'})

	local function KeyValueIter(t, prestate)
		if not t then 
			return nil 
		end

		if t.iterator then
			local key, value = t.iterator(t.t, prestate)

			if key then
				return key, value
			end				
		end
		
		tablestack[t] = true
		return nil, nil		
	end

	function NTI:IterateProfessions()
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local data = professionDB
		
		local iterator, t, state = pairs(data)
		
		tbl.iterator   = iterator
		tbl.t          = t
		
		return KeyValueIter, tbl, state
	end
	
	local function KeyOnlyIter(t, prestate)
		if not t then 
			return nil 
		end

		if t.iterator then
			local key = t.iterator(t.t, prestate)

			if key then
				return key
			end				
		end
		
		tablestack[t] = true
		return nil, nil		
	end

	function NTI:IterateNodes(profession)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local data = nodeDB[profession] or {}
		
		local iterator, t, state = pairs(data)
		
		tbl.iterator   = iterator
		tbl.t          = t
		
		return KeyOnlyIter, tbl, state
	end
	
	local function NodesForExpansionIter(t, prestate)
		if not t then 
			return nil 
		end

		if t.iterator and t.expansion then
			local id = t.iterator(t.t, prestate)

			while id do
				local expansion = NTI:GetExpansion(t.profession, id)
				
				if expansion == t.expansion then
					return id
				end
				
				id = t.iterator(t.t, id)
			end				
		end
		
		tablestack[t] = true
		return nil, nil		
	end

	function NTI:IterateNodesForExpansion(profession, expansion)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local iterator, t, state = self:IterateNodes(profession)
		
		tbl.iterator   = iterator
		tbl.t          = t
		tbl.profession = profession
		tbl.expansion  = expansion
		
		return NodesForExpansionIter, tbl, state
	end
		
	function NTI:IterateNodesForZone(profession, zone)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local data = zoneDB[profession] and zoneDB[profession][zone] or {}
		
		local iterator, t, state = pairs(data)
		
		tbl.iterator = iterator
		tbl.t        = t
		
		return KeyOnlyIter, tbl, state
	end
	
	local function NodesForContinentIter(t, prestate)
		if not t then 
			return nil 
		end
		
		while t.iterator do
			local id = t.iterator(t.t, prestate)

			if id then
				if t.found[id] then
					prestate = id
				else
					t.found[id] = true
					
					return id
				end
			else
				t.iterator = nil
				
				t.index = t.index + 1
				
				if t.index <= #t.zones then
					local zone = t.zones[t.index]
					local data = zone and zoneDB[t.profession] and zoneDB[t.profession][zone] or {}
					
					t.iterator, t.t, prestate = pairs(data)
				end
			end	
		end
		
		-- release additional temporary tables
		tablestack[t.zones] = true
		tablestack[t.found] = true
		
		tablestack[t] = true
		return nil, nil		
	end

	function NTI:IterateNodesForContinent(profession, continent)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local zones = {GetMapZones(continent)}
		
		local zone = zones[1]
		local data = zone and zoneDB[profession] and zoneDB[profession][zone] or {}
		
		local iterator, t, state = pairs(data)
		
		tbl.iterator   = iterator
		tbl.t          = t
		tbl.profession = profession
		tbl.zones      = zones
		tbl.index      = 1
		tbl.found      = next(tablestack) or {}
		
		return NodesForContinentIter, tbl, state
	end
	
	function NTI:IterateNodesForLoot(profession, item)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local data = lootDB[profession] and lootDB[profession][zone] or {}
		
		local iterator, t, state = pairs(data)
		
		tbl.iterator = iterator
		tbl.t        = t
		
		return KeyOnlyIter, tbl, state
	end
	
	function NTI:IterateZonesForNode(profession, id)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local data = nodeDB[profession] and nodeDB[profession][id] and nodeDB[profession][id]["zones"] or {}
		
		return pairs(data)
	end
	
	function NTI:IterateLootForNode(profession, id)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
		
		local data = nodeDB[profession] and nodeDB[profession][id] and nodeDB[profession][id]["contains"] or {}
		
		local iterator, t, state = pairs(data)
		
		tbl.iterator = iterator
		tbl.t        = t
		
		return KeyOnlyIter, tbl, state
	end

	local function RelevantLootForNodeIter(t, prestate)
		if not t then 
			return nil 
		end

		if t.iterator then
			local item, info = t.iterator(t.t, prestate)
			
			while item do
				if info.rate > 0 or info.rate == -1 then
					return item
				end

				item, info = t.iterator(t.t, item)
			end
		end
		
		tablestack[t] = true
		return nil, nil		
	end

	function NTI:IterateRelevantLootForNode(profession, id)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
				
		local data = nodeDB[profession] and nodeDB[profession][id] and nodeDB[profession][id]["contains"] or {}

		local iterator, t, state = pairs(data)
		
		tbl.iterator   = iterator
		tbl.t          = t
		
		return RelevantLootForNodeIter, tbl, state
	end
	
	local function LootForNodeByRateIter(t, prestate)
		if not t then 
			return nil 
		end

		if t.iterator then
			local item, info = t.iterator(t.t, prestate)
			
			while item do
				if info.rate >= t.min and info.rate >= t.max then
					return item
				end

				item, info = t.iterator(t.t, item)
			end
		end
		
		tablestack[t] = true
		return nil, nil		
	end

	function NTI:IterateLootForNodeByRate(profession, id, min, max)
		local tbl = next(tablestack) or {}		
		tablestack[tbl] = nil
				
		local data = nodeDB[profession] and nodeDB[profession][id] and nodeDB[profession][id]["contains"] or {}

		local iterator, t, state = pairs(data)
		
		min = min or 0
		max = max or 1
		
		if max < min then
			min, max = max, min
		end
	
		tbl.iterator   = iterator
		tbl.t          = t
		t.min          = min < 0 and 0 or min
		t.max          = max > 1 and 1 or max
		
		return RelevantLootForNodeIter, tbl, state
	end
	
end

-- returns list of node ids for profession
function NTI:GetNodes(profession)
	local nodes = {}
	
	for id in self:IterateNodes(profession) do
		tinsert(nodes, id)
	end
	
	return nodes
end

-- returns list of ids that belong to a given expansion
function NTI:GetNodesForExpansion(profession, expansion)
	local nodes = {}
	
	for id in self:IterateNodesForExpansion(profession, expansion) do
		tinsert(nodes, id)
	end
	
	return nodes
end

-- returns a copy list of nodes found in given zone
function NTI:GetNodesForZone(profession, zone)
	local nodes = {}
	
	for id in self:IterateNodesForZone(profession, zone) do
		tinsert(nodes, id)
	end
	
	return nodes
end

-- returns a copy list of nodes found in given continent
function NTI:GetNodesForContinent(profession, continent)
	local nodes = {}
	
	for id in self:IterateNodesForContinent(profession, continent) do
		tinsert(nodes, id)
	end
	
	return nodes
end

-- returns a copy list of nodes that can provide mat as loot
function NTI:GetNodesForLoot(profession, item)
	local nodes = {}
	
	for id in self:IterateNodesForLoot(profession, item) do
		tinsert(nodes, id)
	end
	
	return nodes
end

-- returns whether a node is a quest node
function NTI:IsQuestNode(profession, id)
	local data = nodeDB[profession] and nodeDB[profession][id]
	
	return data and data.quest and true or false
end

-- returns quest id of node
function NTI:GetQuest(profession, id)
	local data = nodeDB[profession] and nodeDB[profession][id]
	
	return data and data.quest
end

-- returns a copy list of zone ids where you can find the node
function NTI:GetZones(profession, id)
	local zones = {}

	for zone in self:IterateZonesForNode(profession, id) do
		tinsert(zones, zone)
	end
		
	return zones
end

-- returns a copy list of nodes matching difficulty for given skill and is in one of the specified zones
function NTI:GetMatchingNodes(profession, skill, difficulties, zones)
	local nodes = {}
	
	local zoneData = zoneDB[profession]
		
	if zoneData and skill then
		selection = {}
	
		if type(difficulties) ~= "table" then
			difficulties = {}
		end
		
		if type(zones) ~= "table" then
			zones = {}
		end
	
		-- filter zones
		for idx, zone in ipairs(zones) do
			local nodes = zoneData[zone] or {}
			
			for id in pairs[nodes] do
				selection[id] = true
			end
		end

		-- filter difficulties
		local lookup = {}
		
		for idx, difficulty in ipairs(difficulties) do
			lookup[difficulty] = true
		end
		
		for id in pairs(selection) do
			local difficulty = self:GetDifficulty(profession, id, skill)
			if lookup[difficulty] then
				tinsert(nodes, id)
			end
		end
	end
		
	return nodes
end

-- returns whether a node spawns in given zone
function NTI:SpawnsInZone(profession, id, zone)
	local data = zoneDB[profession] and zoneDB[profession][zone]
		
	return data and data[id] and true or false
end

-- returns a copy list of ids that make up the loot list
function NTI:GetLootList(profession, id)
	local loot = {}

	for item in self:IterateLootForNode(profession, id) do
		tinsert(loot, item)
	end
		
	return loot
end

-- returns a copy list of ids that make up the loot list where drop rate is not 0.0
-- NOTE: drop rate is limited to 0.01%. anything less is listed but with a drop rate of zero
--       mats with rate of zero are probably incorrect entries in the wowhead db which works as source for the lib data
function NTI:GetRelevantLootList(profession, id)
	local loot = {}

	for item in self:IterateRelevantLootForNode(profession, id) do
		tinsert(loot, item)
	end
		
	return loot
end

-- returns a copy list of ids that make up the loot list limited by minimal and maximal loot rate
function NTI:GetLootListByRate(profession, id, min, max)
	local loot = {}

	for item in self:IterateLootForNodeByRate(profession, id, min, max) do
		tinsert(loot, item)
	end
		
	return loot
end

-- returns whether node contains mat in loot list
function NTI:ContainsLoot(profession, id, item)
	local data = nodeDB[profession] and nodeDB[profession][id]
	
	return data and data.contains[item] and true or false
end

-- returns drop rate and min and max count that drops of material
-- NOTE: drop rate is limited to 0.01%. anything less is listed but with a drop rate of zero
--       mats with rate of zero are probably incorrect entries in the wowhead db which works as source for the lib data
function NTI:GetLootInfo(profession, id, item)
	local rate, minAmount, maxAmount = nil, nil, nil
	
	local data = nodeDB[profession] and nodeDB[profession][id]
	
	if data then
		if data.contains[item] then
			rate      = data.contains[item].rate
			minAmount = data.contains[item].minAmount or 1
			maxAmount = data.contains[item].maxAmount or 1
		end
	end
	
	return rate, minAmount, maxAmount
end

-- returns item id of main loot for node type
function NTI:GetMainLoot(profession, id)
	return nodeDB[profession] and nodeDB[profession][id] and nodeDB[profession][id].mainLoot
end

-- setup data

--[[
node = {
	id    = int,                      -- object id
	name  = string,                   -- name (localized if required bubble lib is available)
	profession = int,                 -- the profession to which this node belongs
	contains = {
		id = {                        -- item id
			rate      = float,        -- 0..1 or -1 if unknown
			minAmount = int,          -- minimal amount (only present if != 1)
			maxAmount = int,          -- maximal amount (only present if != 1)
		}
	},
	mainLoot = int,                   -- id of the main item contained in the node
	zones = {
		name = id,                    -- non-localized zone name -> map id (int)
	}, 
	quest = id,                       -- id of quest - only stored if existing
	expansion = id,                   -- id of expansion
}
]]

do
	local function SetupDBs(profession, data)
		local zoneData = {}
		local lootData = {}

		for id, node in pairs(data) do
			node.profession = profession
			
			-- set up zone db
			for zone, _ in pairs(node.zones) do
				if not zoneData[zone] then
					zoneData[zone] = {}
				end
				
				zoneData[zone][id] = true
			end
			
			local maxLootRate = 0
			local maxLootId = nil
			
			-- set up loot db
			for itemId, item in pairs(node.contains) do
				if not lootData[itemId] then
					lootData[itemId] = {}
				end
				
				lootData[itemId][id] = true
				
				if item.rate > maxLootRate then
					maxLootRate = item.rate
					maxLootId = itemId
				end
			end
			
			node.mainLoot = maxLootId 
		end
		
		-- MAIN DBs
		
		-- assign profession specific dbs
		nodeDB[profession]    = data
		zoneDB[profession]    = zoneData
		lootDB[profession]    = lootData
	end

	-- MINING
	local miningNodes = {}

	do -- node data
		miningNodes[181556] = { -- Adamantite Deposit
			id = 181556,
			name = TMN("Adamantite Deposit"),
			expansion = 1,
			zones = {
				[722] = GetMapNameByID(722), -- Auchindoun: Auchenai Crypts
				[732] = GetMapNameByID(732), -- Auchindoun: Mana-Tombs
				[723] = GetMapNameByID(723), -- Auchindoun: Sethekk Halls
				[724] = GetMapNameByID(724), -- Auchindoun: Shadow Labyrinth
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[728] = GetMapNameByID(728), -- Coilfang Reservoir: The Slave Pens
				[727] = GetMapNameByID(727), -- Coilfang Reservoir: The Steamvault
				[726] = GetMapNameByID(726), -- Coilfang Reservoir: The Underbog
				[499] = GetMapNameByID(499), -- Isle of Quel'Danas
				[477] = GetMapNameByID(477), -- Nagrand
				[479] = GetMapNameByID(479), -- Netherstorm
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[478] = GetMapNameByID(478), -- Terokkar Forest
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[23425] = { -- Adamantite Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[22573] = { -- Mote of Earth
					rate = 0.3034,
					maxAmount = 8,
				},
				[23427] = { -- Eternium Ore
					rate = 0.2994,
					maxAmount = 5,
				},
				[35229] = { -- Nether Residue
					rate = 0.0127,
					maxAmount = 2,
				},
				[21929] = { -- Flame Spessarite
					rate = 0.0077,
				},
				[23079] = { -- Deep Peridot
					rate = 0.0074,
				},
				[23112] = { -- Golden Draenite
					rate = 0.0073,
				},
				[23117] = { -- Azure Moonstone
					rate = 0.0073,
				},
				[23107] = { -- Shadow Draenite
					rate = 0.0072,
				},
				[23077] = { -- Blood Garnet
					rate = 0.0072,
				},
				[108302] = { -- Adamantite Ore Nugget
					rate = 0.0042,
					minAmount = 4,
					maxAmount = 20,
				},
				[23440] = { -- Dawnstone
					rate = 0.0025,
				},
				[23439] = { -- Noble Topaz
					rate = 0.0025,
				},
				[23437] = { -- Talasite
					rate = 0.0025,
				},
				[23436] = { -- Living Ruby
					rate = 0.0024,
				},
				[23441] = { -- Nightseye
					rate = 0.0024,
				},
				[23438] = { -- Star of Elune
					rate = 0.0024,
				},
			},
		}

		miningNodes[185557] = { -- Ancient Gem Vein
			id = 185557,
			name = TMN("Ancient Gem Vein"),
			expansion = 1,
			zones = {
				[775] = GetMapNameByID(775), -- Caverns of Time: Hyjal Summit
			},
			contains = {
				[34907] = { -- Shattered Gem Fragments
					rate = 1,
					maxAmount = 9,
				},
				[32227] = { -- Crimson Spinel
					rate = 0.1806,
					maxAmount = 2,
				},
				[32229] = { -- Lionseye
					rate = 0.1768,
					maxAmount = 3,
				},
				[32230] = { -- Shadowsong Amethyst
					rate = 0.1759,
					maxAmount = 2,
				},
				[32231] = { -- Pyrestone
					rate = 0.1758,
					maxAmount = 2,
				},
				[32228] = { -- Empyrean Sapphire
					rate = 0.1749,
					maxAmount = 2,
				},
				[32249] = { -- Seaspray Emerald
					rate = 0.1743,
					maxAmount = 2,
				},
				[23437] = { -- Talasite
					rate = 0.1017,
					maxAmount = 2,
				},
				[23436] = { -- Living Ruby
					rate = 0.0971,
					maxAmount = 2,
				},
				[23440] = { -- Dawnstone
					rate = 0.0922,
					maxAmount = 2,
				},
				[23439] = { -- Noble Topaz
					rate = 0.0911,
				},
				[23441] = { -- Nightseye
					rate = 0.0907,
					maxAmount = 2,
				},
				[23438] = { -- Star of Elune
					rate = 0.0775,
					maxAmount = 2,
				},
			},
		}

		miningNodes[228563] = { -- Blackrock Deposit
			id = 228563,
			name = TMN("Blackrock Deposit"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[976] = GetMapNameByID(976), -- Frostwall
				[949] = GetMapNameByID(949), -- Gorgrond
				[971] = GetMapNameByID(971), -- Lunarfall
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[109118] = { -- Blackrock Ore
					rate = 0.9704,
					maxAmount = 15,
				},
				[115508] = { -- Draenic Stone
					rate = 0.939,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.1489,
					maxAmount = 3,
				},
				[109992] = { -- Blackrock Fragment
					rate = 0.0296,
					maxAmount = 10,
				},
				[111349] = { -- A Treatise on Mining in Draenor
					rate = 0.0036,
				},
			},
		}

		miningNodes[189978] = { -- Cobalt Deposit
			id = 189978,
			name = TMN("Cobalt Deposit"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
				[510] = GetMapNameByID(510), -- Crystalsong Forest
				[488] = GetMapNameByID(488), -- Dragonblight
				[490] = GetMapNameByID(490), -- Grizzly Hills
				[491] = GetMapNameByID(491), -- Howling Fjord
				[495] = GetMapNameByID(495), -- The Storm Peaks
				[523] = GetMapNameByID(523), -- Utgarde Keep: Utgarde Keep
				[496] = GetMapNameByID(496), -- Zul'Drak
			},
			contains = {
				[36909] = { -- Cobalt Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[37701] = { -- Crystallized Earth
					rate = 0.2966,
					maxAmount = 2,
				},
				[37705] = { -- Crystallized Water
					rate = 0.2951,
					maxAmount = 2,
				},
				[36932] = { -- Dark Jade
					rate = 0.0051,
				},
				[36929] = { -- Huge Citrine
					rate = 0.005,
				},
				[36917] = { -- Bloodstone
					rate = 0.005,
				},
				[36926] = { -- Shadow Crystal
					rate = 0.005,
				},
				[36920] = { -- Sun Crystal
					rate = 0.0049,
				},
				[36923] = { -- Chalcedony
					rate = 0.0045,
				},
				[108305] = { -- Cobalt Ore Nugget
					rate = 0.0028,
					minAmount = 4,
					maxAmount = 20,
				},
				[36933] = { -- Forest Emerald
					rate = 0.0026,
				},
				[36924] = { -- Sky Sapphire
					rate = 0.0025,
				},
				[36927] = { -- Twilight Opal
					rate = 0.0025,
				},
				[36918] = { -- Scarlet Ruby
					rate = 0.0025,
				},
				[36930] = { -- Monarch Topaz
					rate = 0.0024,
				},
				[36921] = { -- Autumn's Glow
					rate = 0.0023,
				},
			},
		}

		miningNodes[1731] = { -- Copper Vein
			id = 1731,
			name = TMN("Copper Vein"),
			expansion = 0,
			zones = {
				[43] = GetMapNameByID(43), -- Ashenvale
				[181] = GetMapNameByID(181), -- Azshara
				[464] = GetMapNameByID(464), -- Azuremyst Isle
				[17] = GetMapNameByID(17), -- Badlands
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[890] = GetMapNameByID(890), -- Camp Narache
				[42] = GetMapNameByID(42), -- Darkshore
				[27] = GetMapNameByID(27), -- Dun Morogh
				[4] = GetMapNameByID(4), -- Durotar
				[34] = GetMapNameByID(34), -- Duskwood
				[30] = GetMapNameByID(30), -- Elwynn Forest
				[462] = GetMapNameByID(462), -- Eversong Woods
				[463] = GetMapNameByID(463), -- Ghostlands
				[539] = GetMapNameByID(539), -- Gilneas
				[611] = GetMapNameByID(611), -- Gilneas City
				[35] = GetMapNameByID(35), -- Loch Modan
				[9] = GetMapNameByID(9), -- Mulgore
				[895] = GetMapNameByID(895), -- New Tinkertown
				[11] = GetMapNameByID(11), -- Northern Barrens
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[607] = GetMapNameByID(607), -- Southern Barrens
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[756] = GetMapNameByID(756), -- The Deadmines
				[544] = GetMapNameByID(544), -- The Lost Isles
				[808] = GetMapNameByID(808), -- The Wandering Isle
				[61] = GetMapNameByID(61), -- Thousand Needles
				[362] = GetMapNameByID(362), -- Thunder Bluff
				[20] = GetMapNameByID(20), -- Tirisfal Glades
				[749] = GetMapNameByID(749), -- Wailing Caverns
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[2770] = { -- Copper Ore
					rate = 1,
					maxAmount = 9,
				},
				[2835] = { -- Rough Stone
					rate = 0.7993,
					maxAmount = 11,
				},
				[774] = { -- Malachite
					rate = 0.0297,
				},
				[1210] = { -- Shadowgem
					rate = 0.0294,
				},
				[818] = { -- Tigerseye
					rate = 0.0293,
				},
			},
		}

		miningNodes[165658] = { -- Dark Iron Deposit
			id = 165658,
			name = TMN("Dark Iron Deposit"),
			expansion = 0,
			zones = {
				[704] = GetMapNameByID(704), -- Blackrock Depths
				[696] = GetMapNameByID(696), -- Molten Core
			},
			contains = {
				[11370] = { -- Dark Iron Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[9262] = { -- Black Vitriol
					rate = 0.0104,
				},
				[11382] = { -- Blood of the Mountain
					rate = 0.0102,
				},
				[11754] = { -- Black Diamond
					rate = 0.005,
				},
			},
		}

		miningNodes[202738] = { -- Elementium Vein
			id = 202738,
			name = TMN("Elementium Vein"),
			expansion = 3,
			zones = {
				[640] = GetMapNameByID(640), -- Deepholm
				[708] = GetMapNameByID(708), -- Tol Barad
				[709] = GetMapNameByID(709), -- Tol Barad Peninsula
				[700] = GetMapNameByID(700), -- Twilight Highlands
				[720] = GetMapNameByID(720), -- Uldum
			},
			contains = {
				[52185] = { -- Elementium Ore
					rate = 1,
					maxAmount = 9,
				},
				[52325] = { -- Volatile Fire
					rate = 0.0975,
					maxAmount = 2,
				},
				[52327] = { -- Volatile Earth
					rate = 0.0969,
					maxAmount = 2,
				},
				[52326] = { -- Volatile Water
					rate = 0.0967,
					maxAmount = 2,
				},
				[52328] = { -- Volatile Air
					rate = 0.0242,
					maxAmount = 2,
				},
				[52179] = { -- Alicite
					rate = 0.0083,
				},
				[52177] = { -- Carnelian
					rate = 0.0083,
				},
				[52180] = { -- Nightstone
					rate = 0.0082,
				},
				[52182] = { -- Jasper
					rate = 0.0082,
				},
				[52181] = { -- Hessonite
					rate = 0.0082,
				},
				[52178] = { -- Zephyrite
					rate = 0.0079,
				},
				[52193] = { -- Ember Topaz
					rate = 0.0017,
				},
				[52190] = { -- Inferno Ruby
					rate = 0.0016,
				},
				[52195] = { -- Amberjewel
					rate = 0.0016,
				},
				[52191] = { -- Ocean Sapphire
					rate = 0.0016,
				},
				[52192] = { -- Dream Emerald
					rate = 0.0016,
				},
				[52194] = { -- Demonseye
					rate = 0.0016,
				},
				[67282] = { -- Elementium Geode
					rate = 0.0005,
				},
				[108308] = { -- Elementium Ore Nugget
					rate = 0.0002,
					maxAmount = 20,
				},
			},
		}

		miningNodes[181555] = { -- Fel Iron Deposit
			id = 181555,
			name = TMN("Fel Iron Deposit"),
			expansion = 1,
			zones = {
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[727] = GetMapNameByID(727), -- Coilfang Reservoir: The Steamvault
				[465] = GetMapNameByID(465), -- Hellfire Peninsula
				[477] = GetMapNameByID(477), -- Nagrand
				[479] = GetMapNameByID(479), -- Netherstorm
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[478] = GetMapNameByID(478), -- Terokkar Forest
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[23424] = { -- Fel Iron Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[22574] = { -- Mote of Fire
					rate = 0.2944,
					maxAmount = 2,
				},
				[22573] = { -- Mote of Earth
					rate = 0.2929,
					maxAmount = 2,
				},
				[23427] = { -- Eternium Ore
					rate = 0.0596,
					maxAmount = 5,
				},
				[35229] = { -- Nether Residue
					rate = 0.0067,
					maxAmount = 2,
				},
				[21929] = { -- Flame Spessarite
					rate = 0.0038,
				},
				[23079] = { -- Deep Peridot
					rate = 0.0038,
				},
				[23112] = { -- Golden Draenite
					rate = 0.0037,
				},
				[23107] = { -- Shadow Draenite
					rate = 0.0036,
				},
				[23077] = { -- Blood Garnet
					rate = 0.0036,
				},
				[23117] = { -- Azure Moonstone
					rate = 0.0035,
				},
				[108301] = { -- Fel Iron Ore Nugget
					rate = 0.0035,
					minAmount = 5,
					maxAmount = 20,
				},
				[23437] = { -- Talasite
					rate = 0.0014,
				},
				[23439] = { -- Noble Topaz
					rate = 0.0013,
				},
				[23441] = { -- Nightseye
					rate = 0.0012,
				},
				[23440] = { -- Dawnstone
					rate = 0.0012,
				},
				[23436] = { -- Living Ruby
					rate = 0.0012,
				},
				[23438] = { -- Star of Elune
					rate = 0.0011,
				},
			},
		}

		miningNodes[209311] = { -- Ghost Iron Deposit
			id = 209311,
			name = TMN("Ghost Iron Deposit"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[877] = GetMapNameByID(877), -- Shado-Pan Monastery
				[806] = GetMapNameByID(806), -- The Jade Forest
				[873] = GetMapNameByID(873), -- The Veiled Stair
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
				[811] = GetMapNameByID(811), -- Vale of Eternal Blossoms
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[72092] = { -- Ghost Iron Ore
					rate = 1,
					maxAmount = 9,
				},
				[97512] = { -- Ghost Iron Nugget
					rate = 0.0248,
					maxAmount = 14,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0027,
				},
				[76137] = { -- Alexandrite
					rate = 0.0002,
				},
				[76133] = { -- Lapis Lazuli
					rate = 0.0002,
				},
				[76135] = { -- Roguestone
					rate = 0.0002,
				},
				[76134] = { -- Sunstone
					rate = 0.0002,
				},
				[76136] = { -- Pandarian Garnet
					rate = 0.0002,
				},
				[76130] = { -- Tiger Opal
					rate = 0.0002,
				},
				[76141] = { -- Imperial Amethyst
					rate = 0.0001,
				},
				[76138] = { -- River's Heart
					rate = 0.0001,
				},
				[76131] = { -- Primordial Ruby
					rate = 0.0001,
				},
				[76139] = { -- Wild Jade
					rate = 0.0001,
				},
				[76140] = { -- Vermilion Onyx
					rate = 0.0001,
				},
				[76142] = { -- Sun's Radiance
					rate = 0.0001,
				},
			},
		}

		miningNodes[1734] = { -- Gold Vein
			id = 1734,
			name = TMN("Gold Vein"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[17] = GetMapNameByID(17), -- Badlands
				[29] = GetMapNameByID(29), -- Burning Steppes
				[101] = GetMapNameByID(101), -- Desolace
				[34] = GetMapNameByID(34), -- Duskwood
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[182] = GetMapNameByID(182), -- Felwood
				[121] = GetMapNameByID(121), -- Feralas
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[28] = GetMapNameByID(28), -- Searing Gorge
				[607] = GetMapNameByID(607), -- Southern Barrens
				[161] = GetMapNameByID(161), -- Tanaris
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[61] = GetMapNameByID(61), -- Thousand Needles
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[22] = GetMapNameByID(22), -- Western Plaguelands
			},
			contains = {
				[2776] = { -- Gold Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[3864] = { -- Citrine
					rate = 0.03,
				},
				[1529] = { -- Jade
					rate = 0.0299,
				},
				[1705] = { -- Lesser Moonstone
					rate = 0.0291,
				},
				[108296] = { -- Gold Ore Nugget
					rate = 0.0018,
					minAmount = 9,
					maxAmount = 20,
				},
			},
		}

		miningNodes[1610] = { -- Incendicite Mineral Vein
			id = 1610,
			name = TMN("Incendicite Mineral Vein"),
			expansion = 0,
			zones = {
				[40] = GetMapNameByID(40), -- Wetlands
			},
			quest = 25735, -- Incendicite Ore
			contains = {
				[3340] = { -- Incendicite Ore
					rate = 1,
					maxAmount = 2,
				},
			},
		}

		miningNodes[1735] = { -- Iron Deposit
			id = 1735,
			name = TMN("Iron Deposit"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[17] = GetMapNameByID(17), -- Badlands
				[101] = GetMapNameByID(101), -- Desolace
				[34] = GetMapNameByID(34), -- Duskwood
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[121] = GetMapNameByID(121), -- Feralas
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[761] = GetMapNameByID(761), -- Razorfen Kraul
				[607] = GetMapNameByID(607), -- Southern Barrens
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[26] = GetMapNameByID(26), -- The Hinterlands
				[851] = GetMapNameByID(851), -- Theramore's Fall (H)
				[61] = GetMapNameByID(61), -- Thousand Needles
				[692] = GetMapNameByID(692), -- Uldaman
				[22] = GetMapNameByID(22), -- Western Plaguelands
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[2772] = { -- Iron Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[2838] = { -- Heavy Stone
					rate = 1,
					maxAmount = 10,
				},
				[1529] = { -- Jade
					rate = 0.0396,
				},
				[1705] = { -- Lesser Moonstone
					rate = 0.0294,
				},
				[3864] = { -- Citrine
					rate = 0.0287,
				},
				[7909] = { -- Aquamarine
					rate = 0.0094,
				},
				[108297] = { -- Iron Ore Nugget
					rate = 0.0029,
					minAmount = 9,
					maxAmount = 20,
				},
			},
		}

		miningNodes[181557] = { -- Khorium Vein
			id = 181557,
			name = TMN("Khorium Vein"),
			expansion = 1,
			zones = {
				[722] = GetMapNameByID(722), -- Auchindoun: Auchenai Crypts
				[732] = GetMapNameByID(732), -- Auchindoun: Mana-Tombs
				[724] = GetMapNameByID(724), -- Auchindoun: Shadow Labyrinth
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[499] = GetMapNameByID(499), -- Isle of Quel'Danas
				[477] = GetMapNameByID(477), -- Nagrand
				[479] = GetMapNameByID(479), -- Netherstorm
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[478] = GetMapNameByID(478), -- Terokkar Forest
			},
			contains = {
				[22574] = { -- Mote of Fire
					rate = 1,
					maxAmount = 5,
				},
				[23426] = { -- Khorium Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[22573] = { -- Mote of Earth
					rate = 1,
					maxAmount = 5,
				},
				[23427] = { -- Eternium Ore
					rate = 0.2977,
					maxAmount = 6,
				},
				[21929] = { -- Flame Spessarite
					rate = 0.0654,
				},
				[23079] = { -- Deep Peridot
					rate = 0.064,
				},
				[23077] = { -- Blood Garnet
					rate = 0.062,
				},
				[23112] = { -- Golden Draenite
					rate = 0.0613,
				},
				[23117] = { -- Azure Moonstone
					rate = 0.0613,
				},
				[23107] = { -- Shadow Draenite
					rate = 0.0591,
				},
				[23439] = { -- Noble Topaz
					rate = 0.0222,
				},
				[23440] = { -- Dawnstone
					rate = 0.0219,
				},
				[23438] = { -- Star of Elune
					rate = 0.0212,
				},
				[23437] = { -- Talasite
					rate = 0.0198,
				},
				[23441] = { -- Nightseye
					rate = 0.0195,
				},
				[23436] = { -- Living Ruby
					rate = 0.0194,
				},
				[35229] = { -- Nether Residue
					rate = 0.0122,
					minAmount = 4,
					maxAmount = 5,
				},
				[108304] = { -- Khorium Ore Nugget
					rate = 0.0063,
					minAmount = 4,
					maxAmount = 20,
				},
			},
		}

		miningNodes[209312] = { -- Kyparite Deposit
			id = 209312,
			name = TMN("Kyparite Deposit"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[887] = GetMapNameByID(887), -- Siege of Niuzao Temple
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[72093] = { -- Kyparite
					rate = 1,
					maxAmount = 9,
				},
				[97546] = { -- Kyparite Fragment
					rate = 0.0172,
					maxAmount = 11,
				},
				[76137] = { -- Alexandrite
					rate = 0.0003,
				},
				[76136] = { -- Pandarian Garnet
					rate = 0.0002,
				},
				[76133] = { -- Lapis Lazuli
					rate = 0.0002,
				},
				[76134] = { -- Sunstone
					rate = 0.0002,
				},
				[76140] = { -- Vermilion Onyx
					rate = 0.0002,
				},
				[76135] = { -- Roguestone
					rate = 0.0001,
				},
				[76131] = { -- Primordial Ruby
					rate = 0.0001,
				},
				[76130] = { -- Tiger Opal
					rate = 0.0001,
				},
				[76142] = { -- Sun's Radiance
					rate = 0.0001,
				},
				[76138] = { -- River's Heart
					rate = 0.0001,
				},
				[76141] = { -- Imperial Amethyst
					rate = 0.0001,
				},
				[76139] = { -- Wild Jade
					rate = 0.0001,
				},
			},
		}

		miningNodes[181069] = { -- Large Obsidian Chunk
			id = 181069,
			name = TMN("Large Obsidian Chunk"),
			expansion = 0,
			zones = {
				[717] = GetMapNameByID(717), -- Ruins of Ahn'Qiraj
				[766] = GetMapNameByID(766), -- Temple of Ahn'Qiraj
			},
			contains = {
				[22202] = { -- Small Obsidian Shard
					rate = 0.4522,
					minAmount = 2,
					maxAmount = 7,
				},
				[22203] = { -- Large Obsidian Shard
					rate = 0.3634,
					maxAmount = 5,
				},
				[7076] = { -- Essence of Earth
					rate = 0.0761,
				},
				[12364] = { -- Huge Emerald
					rate = 0.0075,
				},
				[12800] = { -- Azerothian Diamond
					rate = 0.0059,
				},
				[12363] = { -- Arcane Crystal
					rate = 0.0054,
				},
			},
		}

		miningNodes[2653] = { -- Lesser Bloodstone Deposit
			id = 2653,
			name = TMN("Lesser Bloodstone Deposit"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
			},
			quest = 627, -- Favor for Krazek
			contains = {
				[4278] = { -- Lesser Bloodstone Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 3,
				},
			},
		}

		miningNodes[2040] = { -- Mithril Deposit
			id = 2040,
			name = TMN("Mithril Deposit"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[17] = GetMapNameByID(17), -- Badlands
				[29] = GetMapNameByID(29), -- Burning Steppes
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[182] = GetMapNameByID(182), -- Felwood
				[121] = GetMapNameByID(121), -- Feralas
				[750] = GetMapNameByID(750), -- Maraudon
				[28] = GetMapNameByID(28), -- Searing Gorge
				[161] = GetMapNameByID(161), -- Tanaris
				[851] = GetMapNameByID(851), -- Theramore's Fall (H)
				[61] = GetMapNameByID(61), -- Thousand Needles
				[692] = GetMapNameByID(692), -- Uldaman
				[201] = GetMapNameByID(201), -- Un'Goro Crater
			},
			contains = {
				[3858] = { -- Mithril Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[7912] = { -- Solid Stone
					rate = 0.7971,
					maxAmount = 13,
				},
				[3864] = { -- Citrine
					rate = 0.03,
				},
				[7909] = { -- Aquamarine
					rate = 0.0292,
				},
				[7910] = { -- Star Ruby
					rate = 0.029,
				},
				[9262] = { -- Black Vitriol
					rate = 0.0284,
				},
				[108300] = { -- Mithril Ore Nugget
					rate = 0.0027,
					minAmount = 9,
					maxAmount = 20,
				},
				[11513] = { -- Tainted Vitriol
					rate = 0.0002,
				},
				[2771] = { -- Tin Ore
					rate = 0.0001,
					minAmount = 2,
					maxAmount = 4,
				},
			},
		}

		miningNodes[185877] = { -- Nethercite Deposit
			id = 185877,
			name = TMN("Nethercite Deposit"),
			expansion = 1,
			zones = {
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
			},
			quest = 11018, -- Nethercite Ore
			contains = {
				[32464] = { -- Nethercite Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 4,
				},
				[22574] = { -- Mote of Fire
					rate = 0.102,
					minAmount = 2,
					maxAmount = 6,
				},
				[22573] = { -- Mote of Earth
					rate = 0.0976,
					minAmount = 2,
					maxAmount = 6,
				},
				[23427] = { -- Eternium Ore
					rate = 0.0591,
					maxAmount = 4,
				},
				[21929] = { -- Flame Spessarite
					rate = 0.0134,
				},
				[23117] = { -- Azure Moonstone
					rate = 0.0126,
				},
				[23077] = { -- Blood Garnet
					rate = 0.0123,
				},
				[23079] = { -- Deep Peridot
					rate = 0.0122,
				},
				[23112] = { -- Golden Draenite
					rate = 0.0122,
				},
				[23107] = { -- Shadow Draenite
					rate = 0.0119,
				},
				[32506] = { -- Netherwing Egg
					rate = 0.0105,
				},
				[23439] = { -- Noble Topaz
					rate = 0.0045,
				},
				[23441] = { -- Nightseye
					rate = 0.0043,
				},
				[23437] = { -- Talasite
					rate = 0.0041,
				},
				[23436] = { -- Living Ruby
					rate = 0.004,
				},
				[23440] = { -- Dawnstone
					rate = 0.004,
				},
				[23438] = { -- Star of Elune
					rate = 0.0035,
				},
			},
		}

		miningNodes[202736] = { -- Obsidium Deposit
			id = 202736,
			name = TMN("Obsidium Deposit"),
			expansion = 3,
			zones = {
				[640] = GetMapNameByID(640), -- Deepholm
				[606] = GetMapNameByID(606), -- Mount Hyjal
				[614] = GetMapNameByID(614), -- Vashj'ir: Abyssal Depths
				[610] = GetMapNameByID(610), -- Vashj'ir: Kelp'thar Forest
				[615] = GetMapNameByID(615), -- Vashj'ir: Shimmering Expanse
			},
			contains = {
				[53038] = { -- Obsidium Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[52327] = { -- Volatile Earth
					rate = 0.0979,
					maxAmount = 2,
				},
				[52328] = { -- Volatile Air
					rate = 0.0179,
					maxAmount = 2,
				},
				[52182] = { -- Jasper
					rate = 0.0084,
				},
				[52180] = { -- Nightstone
					rate = 0.0082,
				},
				[52181] = { -- Hessonite
					rate = 0.0082,
				},
				[52177] = { -- Carnelian
					rate = 0.0082,
				},
				[52178] = { -- Zephyrite
					rate = 0.0081,
				},
				[52179] = { -- Alicite
					rate = 0.0079,
				},
				[52194] = { -- Demonseye
					rate = 0.0017,
				},
				[52193] = { -- Ember Topaz
					rate = 0.0017,
				},
				[52191] = { -- Ocean Sapphire
					rate = 0.0017,
				},
				[52192] = { -- Dream Emerald
					rate = 0.0016,
				},
				[52190] = { -- Inferno Ruby
					rate = 0.0016,
				},
				[52195] = { -- Amberjewel
					rate = 0.0016,
				},
				[108307] = { -- Obsidium Ore Nugget
					rate = 0.0006,
					minAmount = 3,
					maxAmount = 20,
				},
			},
		}

		miningNodes[123310] = { -- Ooze Covered Mithril Deposit
			id = 123310,
			name = TMN("Ooze Covered Mithril Deposit"),
			expansion = 0,
			zones = {
				[121] = GetMapNameByID(121), -- Feralas
				[61] = GetMapNameByID(61), -- Thousand Needles
			},
			contains = {
				[3858] = { -- Mithril Ore
					rate = 0.9744,
					minAmount = 2,
					maxAmount = 4,
				},
				[7912] = { -- Solid Stone
					rate = 0.6795,
					minAmount = 2,
					maxAmount = 7,
				},
			},
		}

		miningNodes[177388] = { -- Ooze Covered Rich Thorium Vein
			id = 177388,
			name = TMN("Ooze Covered Rich Thorium Vein"),
			expansion = 0,
			zones = {
				[261] = GetMapNameByID(261), -- Silithus
			},
			contains = {
				[10620] = { -- Thorium Ore
					rate = 1,
					minAmount = 4,
					maxAmount = 5,
				},
				[12365] = { -- Dense Stone
					rate = 1,
					minAmount = 4,
					maxAmount = 7,
				},
				[7910] = { -- Star Ruby
					rate = 0.0909,
				},
				[12361] = { -- Blue Sapphire
					rate = 0.0843,
				},
				[12363] = { -- Arcane Crystal
					rate = 0.0554,
				},
				[12800] = { -- Azerothian Diamond
					rate = 0.051,
				},
				[12364] = { -- Huge Emerald
					rate = 0.031,
				},
				[12799] = { -- Large Opal
					rate = 0.0089,
				},
			},
		}

		miningNodes[123848] = { -- Ooze Covered Thorium Vein
			id = 123848,
			name = TMN("Ooze Covered Thorium Vein"),
			expansion = 0,
			zones = {
				[201] = GetMapNameByID(201), -- Un'Goro Crater
			},
			contains = {
				[10620] = { -- Thorium Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 7,
				},
				[12365] = { -- Dense Stone
					rate = 1,
					maxAmount = 8,
				},
				[12363] = { -- Arcane Crystal
					rate = 0.0792,
				},
				[7910] = { -- Star Ruby
					rate = 0.0304,
				},
				[9262] = { -- Black Vitriol
					rate = 0.0297,
				},
				[12364] = { -- Huge Emerald
					rate = 0.0207,
				},
				[12799] = { -- Large Opal
					rate = 0.0194,
				},
				[12800] = { -- Azerothian Diamond
					rate = 0.0192,
				},
				[12361] = { -- Blue Sapphire
					rate = 0.0185,
				},
				[108298] = { -- Thorium Ore Nugget
					rate = 0.0014,
					minAmount = 8,
					maxAmount = 17,
				},
			},
		}

		miningNodes[123309] = { -- Ooze Covered Truesilver Deposit
			id = 123309,
			name = TMN("Ooze Covered Truesilver Deposit"),
			expansion = 0,
			zones = {
				[261] = GetMapNameByID(261), -- Silithus
				[201] = GetMapNameByID(201), -- Un'Goro Crater
			},
			contains = {
				[7911] = { -- Truesilver Ore
					rate = 0.9673,
					minAmount = 2,
					maxAmount = 5,
				},
				[7910] = { -- Star Ruby
					rate = 0.0388,
				},
				[3864] = { -- Citrine
					rate = 0.0347,
				},
				[7909] = { -- Aquamarine
					rate = 0.0286,
				},
			},
		}

		miningNodes[195036] = { -- Pure Saronite Deposit
			id = 195036,
			name = TMN("Pure Saronite Deposit"),
			expansion = 2,
			zones = {
				[529] = GetMapNameByID(529), -- Ulduar
			},
			contains = {
				[36912] = { -- Saronite Ore
					rate = 1,
					minAmount = 28,
					maxAmount = 33,
				},
				[35627] = { -- Eternal Shadow
					rate = 0.891,
					minAmount = 2,
					maxAmount = 4,
				},
				[35624] = { -- Eternal Earth
					rate = 0.7821,
					minAmount = 2,
					maxAmount = 4,
				},
				[36930] = { -- Monarch Topaz
					rate = 0.4679,
					maxAmount = 3,
				},
				[36921] = { -- Autumn's Glow
					rate = 0.4615,
					maxAmount = 4,
				},
				[36923] = { -- Chalcedony
					rate = 0.2949,
					minAmount = 4,
					maxAmount = 9,
				},
				[36924] = { -- Sky Sapphire
					rate = 0.2821,
					maxAmount = 3,
				},
				[36927] = { -- Twilight Opal
					rate = 0.2436,
					maxAmount = 3,
				},
				[36932] = { -- Dark Jade
					rate = 0.2372,
					minAmount = 4,
					maxAmount = 7,
				},
				[36933] = { -- Forest Emerald
					rate = 0.2051,
					maxAmount = 3,
				},
				[36918] = { -- Scarlet Ruby
					rate = 0.1538,
					maxAmount = 5,
				},
				[36926] = { -- Shadow Crystal
					rate = 0.109,
					minAmount = 4,
					maxAmount = 7,
				},
				[36929] = { -- Huge Citrine
					rate = 0.0962,
					minAmount = 4,
					maxAmount = 7,
				},
				[36917] = { -- Bloodstone
					rate = 0.0769,
					minAmount = 4,
					maxAmount = 6,
				},
				[36920] = { -- Sun Crystal
					rate = 0.0641,
					minAmount = 5,
					maxAmount = 5,
				},
			},
		}

		miningNodes[202737] = { -- Pyrite Deposit
			id = 202737,
			name = TMN("Pyrite Deposit"),
			expansion = 3,
			zones = {
				[700] = GetMapNameByID(700), -- Twilight Highlands
				[720] = GetMapNameByID(720), -- Uldum
			},
			contains = {
				[52183] = { -- Pyrite Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[52325] = { -- Volatile Fire
					rate = 0.1012,
					maxAmount = 2,
				},
				[52328] = { -- Volatile Air
					rate = 0.0664,
					maxAmount = 2,
				},
				[52177] = { -- Carnelian
					rate = 0.0083,
				},
				[52178] = { -- Zephyrite
					rate = 0.0082,
				},
				[52182] = { -- Jasper
					rate = 0.0081,
				},
				[52180] = { -- Nightstone
					rate = 0.0081,
				},
				[52181] = { -- Hessonite
					rate = 0.008,
				},
				[52179] = { -- Alicite
					rate = 0.008,
				},
				[52193] = { -- Ember Topaz
					rate = 0.0018,
				},
				[52192] = { -- Dream Emerald
					rate = 0.0016,
				},
				[52191] = { -- Ocean Sapphire
					rate = 0.0016,
				},
				[52194] = { -- Demonseye
					rate = 0.0015,
				},
				[52195] = { -- Amberjewel
					rate = 0.0015,
				},
				[52190] = { -- Inferno Ruby
					rate = 0.0015,
				},
				[108309] = { -- Pyrite Ore Nugget
					rate = 0.0004,
					maxAmount = 20,
				},
			},
		}

		miningNodes[181569] = { -- Rich Adamantite Deposit
			id = 181569,
			name = TMN("Rich Adamantite Deposit"),
			expansion = 1,
			zones = {
				[723] = GetMapNameByID(723), -- Auchindoun: Sethekk Halls
				[724] = GetMapNameByID(724), -- Auchindoun: Shadow Labyrinth
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[499] = GetMapNameByID(499), -- Isle of Quel'Danas
				[477] = GetMapNameByID(477), -- Nagrand
				[479] = GetMapNameByID(479), -- Netherstorm
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[478] = GetMapNameByID(478), -- Terokkar Forest
			},
			contains = {
				[23425] = { -- Adamantite Ore
					rate = 1,
					minAmount = 5,
					maxAmount = 12,
				},
				[22573] = { -- Mote of Earth
					rate = 0.8094,
					maxAmount = 8,
				},
				[23427] = { -- Eternium Ore
					rate = 0.5031,
					maxAmount = 7,
				},
				[23079] = { -- Deep Peridot
					rate = 0.0167,
				},
				[23077] = { -- Blood Garnet
					rate = 0.0166,
				},
				[21929] = { -- Flame Spessarite
					rate = 0.0165,
				},
				[23117] = { -- Azure Moonstone
					rate = 0.0163,
				},
				[23112] = { -- Golden Draenite
					rate = 0.0159,
				},
				[23107] = { -- Shadow Draenite
					rate = 0.0151,
				},
				[35229] = { -- Nether Residue
					rate = 0.0136,
					maxAmount = 4,
				},
				[23437] = { -- Talasite
					rate = 0.0055,
				},
				[23440] = { -- Dawnstone
					rate = 0.0054,
				},
				[108302] = { -- Adamantite Ore Nugget
					rate = 0.0053,
					minAmount = 8,
					maxAmount = 40,
				},
				[23439] = { -- Noble Topaz
					rate = 0.0053,
				},
				[23441] = { -- Nightseye
					rate = 0.0051,
				},
				[23436] = { -- Living Ruby
					rate = 0.0049,
				},
				[23438] = { -- Star of Elune
					rate = 0.0048,
				},
			},
		}

		miningNodes[228564] = { -- Rich Blackrock Deposit
			id = 228564,
			name = TMN("Rich Blackrock Deposit"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[976] = GetMapNameByID(976), -- Frostwall
				[949] = GetMapNameByID(949), -- Gorgrond
				[971] = GetMapNameByID(971), -- Lunarfall
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[109118] = { -- Blackrock Ore
					rate = 1,
					maxAmount = 4,
				},
				[115508] = { -- Draenic Stone
					rate = 1,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.276,
					maxAmount = 3,
				},
				[109992] = { -- Blackrock Fragment
					rate = 0.0055,
					minAmount = 2,
					maxAmount = 9,
				},
				[111349] = { -- A Treatise on Mining in Draenor
					rate = 0.0004,
				},
			},
		}

		miningNodes[189979] = { -- Rich Cobalt Deposit
			id = 189979,
			name = TMN("Rich Cobalt Deposit"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
				[510] = GetMapNameByID(510), -- Crystalsong Forest
				[488] = GetMapNameByID(488), -- Dragonblight
				[490] = GetMapNameByID(490), -- Grizzly Hills
				[491] = GetMapNameByID(491), -- Howling Fjord
				[495] = GetMapNameByID(495), -- The Storm Peaks
				[523] = GetMapNameByID(523), -- Utgarde Keep: Utgarde Keep
				[496] = GetMapNameByID(496), -- Zul'Drak
			},
			contains = {
				[36909] = { -- Cobalt Ore
					rate = 1,
					minAmount = 5,
					maxAmount = 12,
				},
				[37705] = { -- Crystallized Water
					rate = 0.6505,
					maxAmount = 2,
				},
				[37701] = { -- Crystallized Earth
					rate = 0.6476,
					maxAmount = 2,
				},
				[36926] = { -- Shadow Crystal
					rate = 0.0166,
				},
				[36917] = { -- Bloodstone
					rate = 0.0165,
				},
				[36929] = { -- Huge Citrine
					rate = 0.0153,
				},
				[36932] = { -- Dark Jade
					rate = 0.0149,
				},
				[36923] = { -- Chalcedony
					rate = 0.0144,
				},
				[36920] = { -- Sun Crystal
					rate = 0.0143,
				},
				[36921] = { -- Autumn's Glow
					rate = 0.0092,
				},
				[36927] = { -- Twilight Opal
					rate = 0.0085,
				},
				[36918] = { -- Scarlet Ruby
					rate = 0.0085,
				},
				[36924] = { -- Sky Sapphire
					rate = 0.0084,
				},
				[36930] = { -- Monarch Topaz
					rate = 0.0082,
				},
				[36933] = { -- Forest Emerald
					rate = 0.0075,
				},
				[108305] = { -- Cobalt Ore Nugget
					rate = 0.0027,
					minAmount = 8,
					maxAmount = 40,
				},
			},
		}

		miningNodes[202741] = { -- Rich Elementium Vein
			id = 202741,
			name = TMN("Rich Elementium Vein"),
			expansion = 3,
			zones = {
				[640] = GetMapNameByID(640), -- Deepholm
				[708] = GetMapNameByID(708), -- Tol Barad
				[709] = GetMapNameByID(709), -- Tol Barad Peninsula
				[700] = GetMapNameByID(700), -- Twilight Highlands
				[720] = GetMapNameByID(720), -- Uldum
			},
			contains = {
				[52185] = { -- Elementium Ore
					rate = 1,
					minAmount = 5,
					maxAmount = 12,
				},
				[52325] = { -- Volatile Fire
					rate = 0.1955,
					maxAmount = 2,
				},
				[52327] = { -- Volatile Earth
					rate = 0.1946,
					maxAmount = 2,
				},
				[52326] = { -- Volatile Water
					rate = 0.1934,
					maxAmount = 2,
				},
				[52328] = { -- Volatile Air
					rate = 0.0419,
					maxAmount = 2,
				},
				[52182] = { -- Jasper
					rate = 0.0085,
				},
				[52181] = { -- Hessonite
					rate = 0.0083,
				},
				[52177] = { -- Carnelian
					rate = 0.0082,
				},
				[52180] = { -- Nightstone
					rate = 0.0081,
				},
				[52178] = { -- Zephyrite
					rate = 0.0081,
				},
				[52179] = { -- Alicite
					rate = 0.0081,
				},
				[52190] = { -- Inferno Ruby
					rate = 0.0016,
				},
				[52195] = { -- Amberjewel
					rate = 0.0016,
				},
				[52191] = { -- Ocean Sapphire
					rate = 0.0016,
				},
				[52193] = { -- Ember Topaz
					rate = 0.0016,
				},
				[52192] = { -- Dream Emerald
					rate = 0.0016,
				},
				[52194] = { -- Demonseye
					rate = 0.0016,
				},
				[67282] = { -- Elementium Geode
					rate = 0.0008,
				},
				[108308] = { -- Elementium Ore Nugget
					rate = 0.0002,
					minAmount = 2,
					maxAmount = 40,
				},
			},
		}

		miningNodes[209328] = { -- Rich Ghost Iron Deposit
			id = 209328,
			name = TMN("Rich Ghost Iron Deposit"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[877] = GetMapNameByID(877), -- Shado-Pan Monastery
				[806] = GetMapNameByID(806), -- The Jade Forest
				[873] = GetMapNameByID(873), -- The Veiled Stair
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
				[811] = GetMapNameByID(811), -- Vale of Eternal Blossoms
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[72092] = { -- Ghost Iron Ore
					rate = 1,
					maxAmount = 12,
				},
				[97512] = { -- Ghost Iron Nugget
					rate = 0.0222,
					maxAmount = 13,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0028,
				},
				[76135] = { -- Roguestone
					rate = 0.0006,
				},
				[76137] = { -- Alexandrite
					rate = 0.0005,
				},
				[76133] = { -- Lapis Lazuli
					rate = 0.0005,
				},
				[76134] = { -- Sunstone
					rate = 0.0005,
				},
				[76136] = { -- Pandarian Garnet
					rate = 0.0005,
				},
				[76130] = { -- Tiger Opal
					rate = 0.0005,
				},
				[76142] = { -- Sun's Radiance
					rate = 0.0003,
				},
				[76138] = { -- River's Heart
					rate = 0.0003,
				},
				[76131] = { -- Primordial Ruby
					rate = 0.0003,
				},
				[76139] = { -- Wild Jade
					rate = 0.0002,
				},
				[76140] = { -- Vermilion Onyx
					rate = 0.0002,
				},
				[76141] = { -- Imperial Amethyst
					rate = 0.0002,
				},
			},
		}

		miningNodes[209329] = { -- Rich Kyparite Deposit
			id = 209329,
			name = TMN("Rich Kyparite Deposit"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[72093] = { -- Kyparite
					rate = 1,
					maxAmount = 12,
				},
				[97546] = { -- Kyparite Fragment
					rate = 0.015,
					maxAmount = 9,
				},
				[76133] = { -- Lapis Lazuli
					rate = 0.0004,
				},
				[76135] = { -- Roguestone
					rate = 0.0004,
				},
				[76137] = { -- Alexandrite
					rate = 0.0004,
				},
				[76140] = { -- Vermilion Onyx
					rate = 0.0004,
				},
				[76142] = { -- Sun's Radiance
					rate = 0.0003,
				},
				[76134] = { -- Sunstone
					rate = 0.0002,
				},
				[76139] = { -- Wild Jade
					rate = 0.0002,
				},
				[76141] = { -- Imperial Amethyst
					rate = 0.0002,
				},
				[76131] = { -- Primordial Ruby
					rate = 0.0002,
				},
				[76136] = { -- Pandarian Garnet
					rate = 0.0001,
				},
				[76130] = { -- Tiger Opal
					rate = 0.0001,
				},
			},
		}

		miningNodes[202739] = { -- Rich Obsidium Deposit
			id = 202739,
			name = TMN("Rich Obsidium Deposit"),
			expansion = 3,
			zones = {
				[640] = GetMapNameByID(640), -- Deepholm
			},
			contains = {
				[53038] = { -- Obsidium Ore
					rate = 1,
					minAmount = 5,
					maxAmount = 12,
				},
				[52327] = { -- Volatile Earth
					rate = 0.1947,
					maxAmount = 4,
				},
				[52328] = { -- Volatile Air
					rate = 0.0372,
					maxAmount = 2,
				},
				[52179] = { -- Alicite
					rate = 0.0087,
				},
				[52180] = { -- Nightstone
					rate = 0.0085,
				},
				[52181] = { -- Hessonite
					rate = 0.0085,
				},
				[52177] = { -- Carnelian
					rate = 0.0082,
				},
				[52182] = { -- Jasper
					rate = 0.008,
				},
				[52178] = { -- Zephyrite
					rate = 0.0074,
				},
				[52192] = { -- Dream Emerald
					rate = 0.0018,
				},
				[52193] = { -- Ember Topaz
					rate = 0.0017,
				},
				[52194] = { -- Demonseye
					rate = 0.0016,
				},
				[52195] = { -- Amberjewel
					rate = 0.0015,
				},
				[52190] = { -- Inferno Ruby
					rate = 0.0015,
				},
				[52191] = { -- Ocean Sapphire
					rate = 0.0015,
				},
				[108307] = { -- Obsidium Ore Nugget
					rate = 0.0001,
					minAmount = 6,
					maxAmount = 39,
				},
			},
		}

		miningNodes[202740] = { -- Rich Pyrite Deposit
			id = 202740,
			name = TMN("Rich Pyrite Deposit"),
			expansion = 3,
			zones = {
				[708] = GetMapNameByID(708), -- Tol Barad
				[709] = GetMapNameByID(709), -- Tol Barad Peninsula
			},
			contains = {
				[52183] = { -- Pyrite Ore
					rate = 1,
					minAmount = 5,
					maxAmount = 11,
				},
				[52325] = { -- Volatile Fire
					rate = 0.1945,
					maxAmount = 3,
				},
				[52328] = { -- Volatile Air
					rate = 0.1407,
					maxAmount = 2,
				},
				[52181] = { -- Hessonite
					rate = 0.0088,
				},
				[52177] = { -- Carnelian
					rate = 0.0085,
				},
				[52178] = { -- Zephyrite
					rate = 0.008,
				},
				[52180] = { -- Nightstone
					rate = 0.0079,
				},
				[52179] = { -- Alicite
					rate = 0.0077,
				},
				[52182] = { -- Jasper
					rate = 0.0071,
				},
				[52192] = { -- Dream Emerald
					rate = 0.002,
				},
				[52194] = { -- Demonseye
					rate = 0.0018,
				},
				[52191] = { -- Ocean Sapphire
					rate = 0.0017,
				},
				[52195] = { -- Amberjewel
					rate = 0.0016,
				},
				[52190] = { -- Inferno Ruby
					rate = 0.0016,
				},
				[52193] = { -- Ember Topaz
					rate = 0.0015,
				},
			},
		}

		miningNodes[189981] = { -- Rich Saronite Deposit
			id = 189981,
			name = TMN("Rich Saronite Deposit"),
			expansion = 2,
			zones = {
				[510] = GetMapNameByID(510), -- Crystalsong Forest
				[488] = GetMapNameByID(488), -- Dragonblight
				[492] = GetMapNameByID(492), -- Icecrown
				[493] = GetMapNameByID(493), -- Sholazar Basin
				[495] = GetMapNameByID(495), -- The Storm Peaks
				[501] = GetMapNameByID(501), -- Wintergrasp
			},
			contains = {
				[36912] = { -- Saronite Ore
					rate = 1,
					minAmount = 5,
					maxAmount = 12,
				},
				[37703] = { -- Crystallized Shadow
					rate = 0.4989,
					maxAmount = 8,
				},
				[37701] = { -- Crystallized Earth
					rate = 0.4957,
					maxAmount = 8,
				},
				[36932] = { -- Dark Jade
					rate = 0.0837,
				},
				[36923] = { -- Chalcedony
					rate = 0.0835,
				},
				[36929] = { -- Huge Citrine
					rate = 0.0834,
				},
				[36917] = { -- Bloodstone
					rate = 0.083,
				},
				[36926] = { -- Shadow Crystal
					rate = 0.0817,
				},
				[36920] = { -- Sun Crystal
					rate = 0.0813,
				},
				[36921] = { -- Autumn's Glow
					rate = 0.0086,
				},
				[36918] = { -- Scarlet Ruby
					rate = 0.0085,
				},
				[36930] = { -- Monarch Topaz
					rate = 0.0085,
				},
				[36924] = { -- Sky Sapphire
					rate = 0.0083,
				},
				[36927] = { -- Twilight Opal
					rate = 0.0082,
				},
				[36933] = { -- Forest Emerald
					rate = 0.0081,
				},
				[108306] = { -- Saronite Ore Nugget
					rate = 0.0015,
					minAmount = 6,
					maxAmount = 40,
				},
			},
		}

		miningNodes[175404] = { -- Rich Thorium Vein
			id = 175404,
			name = TMN("Rich Thorium Vein"),
			expansion = 0,
			zones = {
				[19] = GetMapNameByID(19), -- Blasted Lands
				[699] = GetMapNameByID(699), -- Dire Maul
				[261] = GetMapNameByID(261), -- Silithus
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[281] = GetMapNameByID(281), -- Winterspring
			},
			contains = {
				[10620] = { -- Thorium Ore
					rate = 1,
					minAmount = 4,
					maxAmount = 10,
				},
				[12365] = { -- Dense Stone
					rate = 1,
					minAmount = 4,
					maxAmount = 12,
				},
				[12363] = { -- Arcane Crystal
					rate = 0.1315,
				},
				[7910] = { -- Star Ruby
					rate = 0.0405,
				},
				[12799] = { -- Large Opal
					rate = 0.0337,
				},
				[12361] = { -- Blue Sapphire
					rate = 0.0331,
				},
				[12800] = { -- Azerothian Diamond
					rate = 0.0328,
				},
				[12364] = { -- Huge Emerald
					rate = 0.0323,
				},
			},
		}

		miningNodes[209330] = { -- Rich Trillium Vein
			id = 209330,
			name = TMN("Rich Trillium Vein"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[806] = GetMapNameByID(806), -- The Jade Forest
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
				[811] = GetMapNameByID(811), -- Vale of Eternal Blossoms
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[72094] = { -- Black Trillium Ore
					rate = 0.5683,
					minAmount = 2,
					maxAmount = 14,
				},
				[72103] = { -- White Trillium Ore
					rate = 0.5675,
					minAmount = 2,
					maxAmount = 14,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0076,
				},
				[76134] = { -- Sunstone
					rate = 0.0035,
				},
				[76135] = { -- Roguestone
					rate = 0.0033,
				},
				[76130] = { -- Tiger Opal
					rate = 0.0032,
				},
				[76136] = { -- Pandarian Garnet
					rate = 0.0031,
				},
				[76137] = { -- Alexandrite
					rate = 0.0031,
				},
				[76133] = { -- Lapis Lazuli
					rate = 0.0028,
				},
				[76140] = { -- Vermilion Onyx
					rate = 0.002,
				},
				[76141] = { -- Imperial Amethyst
					rate = 0.0019,
				},
				[76142] = { -- Sun's Radiance
					rate = 0.0018,
				},
				[76139] = { -- Wild Jade
					rate = 0.0014,
				},
				[76138] = { -- River's Heart
					rate = 0.0014,
				},
				[76131] = { -- Primordial Ruby
					rate = 0.0013,
				},
			},
		}

		miningNodes[228510] = { -- Rich True Iron Deposit
			id = 228510,
			name = TMN("Rich True Iron Deposit"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[976] = GetMapNameByID(976), -- Frostwall
				[949] = GetMapNameByID(949), -- Gorgrond
				[971] = GetMapNameByID(971), -- Lunarfall
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[109119] = { -- True Iron Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 18,
				},
				[115508] = { -- Draenic Stone
					rate = 0.9461,
					maxAmount = 5,
				},
				[120945] = { -- Primal Spirit
					rate = 0.2761,
				},
				[109991] = { -- True Iron Nugget
					rate = 0.0054,
					minAmount = 3,
					maxAmount = 4,
				},
				[111349] = { -- A Treatise on Mining in Draenor
					rate = 0.0004,
				},
			},
		}

		miningNodes[189980] = { -- Saronite Deposit
			id = 189980,
			name = TMN("Saronite Deposit"),
			expansion = 2,
			zones = {
				[510] = GetMapNameByID(510), -- Crystalsong Forest
				[488] = GetMapNameByID(488), -- Dragonblight
				[490] = GetMapNameByID(490), -- Grizzly Hills
				[492] = GetMapNameByID(492), -- Icecrown
				[604] = GetMapNameByID(604), -- Icecrown Citadel
				[493] = GetMapNameByID(493), -- Sholazar Basin
				[495] = GetMapNameByID(495), -- The Storm Peaks
				[526] = GetMapNameByID(526), -- Ulduar: Halls of Stone
				[501] = GetMapNameByID(501), -- Wintergrasp
				[496] = GetMapNameByID(496), -- Zul'Drak
			},
			contains = {
				[36912] = { -- Saronite Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[37701] = { -- Crystallized Earth
					rate = 0.4476,
					maxAmount = 2,
				},
				[37703] = { -- Crystallized Shadow
					rate = 0.4459,
					maxAmount = 2,
				},
				[36929] = { -- Huge Citrine
					rate = 0.0051,
				},
				[36932] = { -- Dark Jade
					rate = 0.005,
				},
				[36926] = { -- Shadow Crystal
					rate = 0.0049,
				},
				[36917] = { -- Bloodstone
					rate = 0.0049,
				},
				[36923] = { -- Chalcedony
					rate = 0.0049,
				},
				[36920] = { -- Sun Crystal
					rate = 0.0047,
				},
				[36927] = { -- Twilight Opal
					rate = 0.0026,
				},
				[36933] = { -- Forest Emerald
					rate = 0.0025,
				},
				[36924] = { -- Sky Sapphire
					rate = 0.0025,
				},
				[36918] = { -- Scarlet Ruby
					rate = 0.0025,
				},
				[36921] = { -- Autumn's Glow
					rate = 0.0025,
				},
				[36930] = { -- Monarch Topaz
					rate = 0.0025,
				},
				[108306] = { -- Saronite Ore Nugget
					rate = 0.0017,
					minAmount = 3,
					maxAmount = 20,
				},
			},
		}

		miningNodes[1733] = { -- Silver Vein
			id = 1733,
			name = TMN("Silver Vein"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[181] = GetMapNameByID(181), -- Azshara
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[101] = GetMapNameByID(101), -- Desolace
				[34] = GetMapNameByID(34), -- Duskwood
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[121] = GetMapNameByID(121), -- Feralas
				[463] = GetMapNameByID(463), -- Ghostlands
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[35] = GetMapNameByID(35), -- Loch Modan
				[11] = GetMapNameByID(11), -- Northern Barrens
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[761] = GetMapNameByID(761), -- Razorfen Kraul
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[607] = GetMapNameByID(607), -- Southern Barrens
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[756] = GetMapNameByID(756), -- The Deadmines
				[26] = GetMapNameByID(26), -- The Hinterlands
				[61] = GetMapNameByID(61), -- Thousand Needles
				[692] = GetMapNameByID(692), -- Uldaman
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[2775] = { -- Silver Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[1206] = { -- Moss Agate
					rate = 0.0491,
				},
				[1705] = { -- Lesser Moonstone
					rate = 0.0301,
				},
				[1210] = { -- Shadowgem
					rate = 0.0291,
				},
				[108294] = { -- Silver Ore Nugget
					rate = 0.0012,
					minAmount = 14,
					maxAmount = 20,
				},
			},
		}

		miningNodes[181068] = { -- Small Obsidian Chunk
			id = 181068,
			name = TMN("Small Obsidian Chunk"),
			expansion = 0,
			zones = {
				[717] = GetMapNameByID(717), -- Ruins of Ahn'Qiraj
				[766] = GetMapNameByID(766), -- Temple of Ahn'Qiraj
			},
			contains = {
				[22202] = { -- Small Obsidian Shard
					rate = 0.8218,
					maxAmount = 6,
				},
				[7076] = { -- Essence of Earth
					rate = 0.0407,
				},
				[22203] = { -- Large Obsidian Shard
					rate = 0.0369,
					maxAmount = 4,
				},
				[12800] = { -- Azerothian Diamond
					rate = 0.0077,
				},
				[12364] = { -- Huge Emerald
					rate = 0.0056,
				},
				[12363] = { -- Arcane Crystal
					rate = 0.0033,
				},
			},
		}

		miningNodes[324] = { -- Small Thorium Vein
			id = 324,
			name = TMN("Small Thorium Vein"),
			expansion = 0,
			zones = {
				[19] = GetMapNameByID(19), -- Blasted Lands
				[182] = GetMapNameByID(182), -- Felwood
				[261] = GetMapNameByID(261), -- Silithus
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[26] = GetMapNameByID(26), -- The Hinterlands
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[281] = GetMapNameByID(281), -- Winterspring
			},
			contains = {
				[10620] = { -- Thorium Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[12365] = { -- Dense Stone
					rate = 1,
					maxAmount = 10,
				},
				[12363] = { -- Arcane Crystal
					rate = 0.0869,
				},
				[7910] = { -- Star Ruby
					rate = 0.0294,
				},
				[9262] = { -- Black Vitriol
					rate = 0.0291,
				},
				[12799] = { -- Large Opal
					rate = 0.0221,
				},
				[12800] = { -- Azerothian Diamond
					rate = 0.022,
				},
				[12361] = { -- Blue Sapphire
					rate = 0.022,
				},
				[12364] = { -- Huge Emerald
					rate = 0.021,
				},
				[108298] = { -- Thorium Ore Nugget
					rate = 0.0024,
					minAmount = 7,
					maxAmount = 20,
				},
				[11513] = { -- Tainted Vitriol
					rate = 0.0002,
				},
			},
		}

		miningNodes[230428] = { -- Smoldering True Iron Deposit
			id = 230428,
			name = TMN("Smoldering True Iron Deposit"),
			expansion = 5,
			zones = {
				[941] = GetMapNameByID(941), -- Frostfire Ridge
			},
			contains = {
				[109119] = { -- True Iron Ore
					rate = 1,
					maxAmount = 4,
				},
				[120945] = { -- Primal Spirit
					rate = 0.6266,
					maxAmount = 3,
				},
			},
		}

		miningNodes[1732] = { -- Tin Vein
			id = 1732,
			name = TMN("Tin Vein"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[181] = GetMapNameByID(181), -- Azshara
				[17] = GetMapNameByID(17), -- Badlands
				[688] = GetMapNameByID(688), -- Blackfathom Deeps
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[34] = GetMapNameByID(34), -- Duskwood
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[463] = GetMapNameByID(463), -- Ghostlands
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[35] = GetMapNameByID(35), -- Loch Modan
				[11] = GetMapNameByID(11), -- Northern Barrens
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[756] = GetMapNameByID(756), -- The Deadmines
				[26] = GetMapNameByID(26), -- The Hinterlands
				[700] = GetMapNameByID(700), -- Twilight Highlands
				[749] = GetMapNameByID(749), -- Wailing Caverns
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[2771] = { -- Tin Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 4,
				},
				[2836] = { -- Coarse Stone
					rate = 0.8014,
					maxAmount = 8,
				},
				[1206] = { -- Moss Agate
					rate = 0.0355,
				},
				[1210] = { -- Shadowgem
					rate = 0.0299,
				},
				[1705] = { -- Lesser Moonstone
					rate = 0.0295,
				},
				[1529] = { -- Jade
					rate = 0.0096,
				},
			},
		}

		miningNodes[191133] = { -- Titanium Vein
			id = 191133,
			name = TMN("Titanium Vein"),
			expansion = 2,
			zones = {
				[510] = GetMapNameByID(510), -- Crystalsong Forest
				[488] = GetMapNameByID(488), -- Dragonblight
				[492] = GetMapNameByID(492), -- Icecrown
				[493] = GetMapNameByID(493), -- Sholazar Basin
				[495] = GetMapNameByID(495), -- The Storm Peaks
				[501] = GetMapNameByID(501), -- Wintergrasp
			},
			contains = {
				[36910] = { -- Titanium Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37700] = { -- Crystallized Air
					rate = 0.4982,
					maxAmount = 6,
				},
				[37702] = { -- Crystallized Fire
					rate = 0.4978,
					maxAmount = 6,
				},
				[37705] = { -- Crystallized Water
					rate = 0.4971,
					maxAmount = 6,
				},
				[37701] = { -- Crystallized Earth
					rate = 0.4934,
					maxAmount = 6,
				},
				[36929] = { -- Huge Citrine
					rate = 0.085,
				},
				[36923] = { -- Chalcedony
					rate = 0.0842,
				},
				[36932] = { -- Dark Jade
					rate = 0.0825,
				},
				[36917] = { -- Bloodstone
					rate = 0.0822,
				},
				[36920] = { -- Sun Crystal
					rate = 0.0815,
				},
				[36926] = { -- Shadow Crystal
					rate = 0.0802,
				},
				[36927] = { -- Twilight Opal
					rate = 0.0174,
				},
				[36918] = { -- Scarlet Ruby
					rate = 0.0169,
				},
				[36924] = { -- Sky Sapphire
					rate = 0.0165,
				},
				[36930] = { -- Monarch Topaz
					rate = 0.0164,
				},
				[36921] = { -- Autumn's Glow
					rate = 0.0162,
				},
				[36933] = { -- Forest Emerald
					rate = 0.016,
				},
				[108391] = { -- Titanium Ore Nugget
					rate = 0.002,
					minAmount = 3,
					maxAmount = 20,
				},
			},
		}

		miningNodes[209313] = { -- Trillium Vein
			id = 209313,
			name = TMN("Trillium Vein"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[806] = GetMapNameByID(806), -- The Jade Forest
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
				[811] = GetMapNameByID(811), -- Vale of Eternal Blossoms
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[72094] = { -- Black Trillium Ore
					rate = 0.5004,
					maxAmount = 8,
				},
				[72103] = { -- White Trillium Ore
					rate = 0.4991,
					maxAmount = 8,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0047,
				},
				[76137] = { -- Alexandrite
					rate = 0.0003,
				},
				[76133] = { -- Lapis Lazuli
					rate = 0.0003,
				},
				[76134] = { -- Sunstone
					rate = 0.0003,
				},
				[76135] = { -- Roguestone
					rate = 0.0003,
				},
				[76136] = { -- Pandarian Garnet
					rate = 0.0003,
				},
				[76130] = { -- Tiger Opal
					rate = 0.0003,
				},
				[76142] = { -- Sun's Radiance
					rate = 0.0002,
				},
				[76139] = { -- Wild Jade
					rate = 0.0002,
				},
				[76131] = { -- Primordial Ruby
					rate = 0.0001,
				},
				[76140] = { -- Vermilion Onyx
					rate = 0.0001,
				},
				[76141] = { -- Imperial Amethyst
					rate = 0.0001,
				},
				[76138] = { -- River's Heart
					rate = 0.0001,
				},
			},
		}

		miningNodes[228493] = { -- True Iron Deposit
			id = 228493,
			name = TMN("True Iron Deposit"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[976] = GetMapNameByID(976), -- Frostwall
				[949] = GetMapNameByID(949), -- Gorgrond
				[971] = GetMapNameByID(971), -- Lunarfall
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[109119] = { -- True Iron Ore
					rate = 0.9713,
					minAmount = 2,
					maxAmount = 8,
				},
				[115508] = { -- Draenic Stone
					rate = 0.7138,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.1496,
				},
				[109991] = { -- True Iron Nugget
					rate = 0.0289,
					maxAmount = 9,
				},
				[111349] = { -- A Treatise on Mining in Draenor
					rate = 0.0038,
				},
			},
		}

		miningNodes[2047] = { -- Truesilver Deposit
			id = 2047,
			name = TMN("Truesilver Deposit"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[17] = GetMapNameByID(17), -- Badlands
				[19] = GetMapNameByID(19), -- Blasted Lands
				[29] = GetMapNameByID(29), -- Burning Steppes
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[182] = GetMapNameByID(182), -- Felwood
				[121] = GetMapNameByID(121), -- Feralas
				[750] = GetMapNameByID(750), -- Maraudon
				[28] = GetMapNameByID(28), -- Searing Gorge
				[261] = GetMapNameByID(261), -- Silithus
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[161] = GetMapNameByID(161), -- Tanaris
				[26] = GetMapNameByID(26), -- The Hinterlands
				[61] = GetMapNameByID(61), -- Thousand Needles
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[281] = GetMapNameByID(281), -- Winterspring
			},
			contains = {
				[7911] = { -- Truesilver Ore
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[7909] = { -- Aquamarine
					rate = 0.0477,
				},
				[7910] = { -- Star Ruby
					rate = 0.0294,
				},
				[3864] = { -- Citrine
					rate = 0.0291,
				},
				[108299] = { -- Truesilver Ore Nugget
					rate = 0.0018,
					minAmount = 7,
					maxAmount = 20,
				},
			},
		}

	end
	
	-- HERBALISM
	local herbalismNodes = {}

	do -- node data
		herbalismNodes[191019] = { -- Adder's Tongue
			id = 191019,
			name = THN("Adder's Tongue"),
			expansion = 2,
			zones = {
				[534] = GetMapNameByID(534), -- Drak'Tharon Keep
				[530] = GetMapNameByID(530), -- Gundrak
				[493] = GetMapNameByID(493), -- Sholazar Basin
				[529] = GetMapNameByID(529), -- Ulduar
			},
			contains = {
				[36903] = { -- Adder's Tongue
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37704] = { -- Crystallized Life
					rate = 0.3083,
					maxAmount = 3,
				},
				[36908] = { -- Frost Lotus
					rate = 0.0765,
					maxAmount = 5,
				},
			},
		}

		herbalismNodes[181278] = { -- Ancient Lichen
			id = 181278,
			name = THN("Ancient Lichen"),
			expansion = 1,
			zones = {
				[722] = GetMapNameByID(722), -- Auchindoun: Auchenai Crypts
				[732] = GetMapNameByID(732), -- Auchindoun: Mana-Tombs
				[723] = GetMapNameByID(723), -- Auchindoun: Sethekk Halls
				[724] = GetMapNameByID(724), -- Auchindoun: Shadow Labyrinth
				[728] = GetMapNameByID(728), -- Coilfang Reservoir: The Slave Pens
				[727] = GetMapNameByID(727), -- Coilfang Reservoir: The Steamvault
				[726] = GetMapNameByID(726), -- Coilfang Reservoir: The Underbog
			},
			contains = {
				[22790] = { -- Ancient Lichen
					rate = 1,
					maxAmount = 8,
				},
				[22575] = { -- Mote of Life
					rate = 0.1647,
					maxAmount = 3,
				},
				[22794] = { -- Fel Lotus
					rate = 0.0543,
					maxAmount = 4,
				},
				[35229] = { -- Nether Residue
					rate = 0.0042,
				},
			},
		}

		herbalismNodes[142141] = { -- Arthas' Tears
			id = 142141,
			name = THN("Arthas' Tears"),
			expansion = 0,
			zones = {
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[182] = GetMapNameByID(182), -- Felwood
				[760] = GetMapNameByID(760), -- Razorfen Downs
				[22] = GetMapNameByID(22), -- Western Plaguelands
			},
			contains = {
				[8836] = { -- Arthas' Tears
					rate = 0.9403,
					maxAmount = 6,
				},
			},
		}

		herbalismNodes[202749] = { -- Azshara's Veil
			id = 202749,
			name = THN("Azshara's Veil"),
			expansion = 3,
			zones = {
				[606] = GetMapNameByID(606), -- Mount Hyjal
				[709] = GetMapNameByID(709), -- Tol Barad Peninsula
				[614] = GetMapNameByID(614), -- Vashj'ir: Abyssal Depths
				[610] = GetMapNameByID(610), -- Vashj'ir: Kelp'thar Forest
				[615] = GetMapNameByID(615), -- Vashj'ir: Shimmering Expanse
			},
			contains = {
				[52985] = { -- Azshara's Veil
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[52329] = { -- Volatile Life
					rate = 0.3885,
					maxAmount = 3,
				},
				[63122] = { -- Lifegiving Seed
					rate = 0.0991,
				},
			},
		}

		herbalismNodes[176589] = { -- Black Lotus
			id = 176589,
			name = THN("Black Lotus"),
			expansion = 0,
			zones = {
				[19] = GetMapNameByID(19), -- Blasted Lands
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[261] = GetMapNameByID(261), -- Silithus
				[281] = GetMapNameByID(281), -- Winterspring
			},
			contains = {
				[13468] = { -- Black Lotus
					rate = 0.9731,
					maxAmount = 3,
				},
			},
		}

		herbalismNodes[142143] = { -- Blindweed
			id = 142143,
			name = THN("Blindweed"),
			expansion = 0,
			zones = {
				[121] = GetMapNameByID(121), -- Feralas
				[750] = GetMapNameByID(750), -- Maraudon
				[26] = GetMapNameByID(26), -- The Hinterlands
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[22] = GetMapNameByID(22), -- Western Plaguelands
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[8839] = { -- Blindweed
					rate = 1,
					maxAmount = 8,
				},
				[24401] = { -- Unidentified Plant Parts
					rate = 0.102,
				},
			},
		}

		herbalismNodes[181166] = { -- Bloodthistle
			id = 181166,
			name = THN("Bloodthistle"),
			expansion = 1,
			zones = {
				[462] = GetMapNameByID(462), -- Eversong Woods
			},
			contains = {
				[22710] = { -- Bloodthistle
					rate = 1,
					maxAmount = 6,
				},
			},
		}

		herbalismNodes[1621] = { -- Briarthorn
			id = 1621,
			name = THN("Briarthorn"),
			expansion = 0,
			zones = {
				[43] = GetMapNameByID(43), -- Ashenvale
				[181] = GetMapNameByID(181), -- Azshara
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[34] = GetMapNameByID(34), -- Duskwood
				[463] = GetMapNameByID(463), -- Ghostlands
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[35] = GetMapNameByID(35), -- Loch Modan
				[11] = GetMapNameByID(11), -- Northern Barrens
				[761] = GetMapNameByID(761), -- Razorfen Kraul
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[607] = GetMapNameByID(607), -- Southern Barrens
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[2450] = { -- Briarthorn
					rate = 1,
					maxAmount = 8,
				},
				[2452] = { -- Swiftthistle
					rate = 0.3999,
					maxAmount = 7,
				},
				[5056] = { -- Root Sample
					rate = 0.0083,
				},
			},
		}

		herbalismNodes[1622] = { -- Bruiseweed
			id = 1622,
			name = THN("Bruiseweed"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[688] = GetMapNameByID(688), -- Blackfathom Deeps
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[34] = GetMapNameByID(34), -- Duskwood
				[463] = GetMapNameByID(463), -- Ghostlands
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[35] = GetMapNameByID(35), -- Loch Modan
				[11] = GetMapNameByID(11), -- Northern Barrens
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[26] = GetMapNameByID(26), -- The Hinterlands
				[749] = GetMapNameByID(749), -- Wailing Caverns
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[2453] = { -- Bruiseweed
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[202747] = { -- Cinderbloom
			id = 202747,
			name = THN("Cinderbloom"),
			expansion = 3,
			zones = {
				[640] = GetMapNameByID(640), -- Deepholm
				[606] = GetMapNameByID(606), -- Mount Hyjal
				[708] = GetMapNameByID(708), -- Tol Barad
				[709] = GetMapNameByID(709), -- Tol Barad Peninsula
				[700] = GetMapNameByID(700), -- Twilight Highlands
				[720] = GetMapNameByID(720), -- Uldum
			},
			contains = {
				[52983] = { -- Cinderbloom
					rate = 1,
					maxAmount = 9,
				},
				[52329] = { -- Volatile Life
					rate = 0.3914,
					maxAmount = 3,
				},
				[63122] = { -- Lifegiving Seed
					rate = 0.0968,
				},
			},
		}

		herbalismNodes[2044] = { -- Dragon's Teeth
			id = 2044,
			name = THN("Dragon's Teeth"),
			expansion = 0,
			zones = {
				[17] = GetMapNameByID(17), -- Badlands
			},
			contains = {
				[3819] = { -- Dragon's Teeth
					rate = 1,
					maxAmount = 7,
				},
			},
		}

		herbalismNodes[176584] = { -- Dreamfoil
			id = 176584,
			name = THN("Dreamfoil"),
			expansion = 0,
			zones = {
				[19] = GetMapNameByID(19), -- Blasted Lands
				[29] = GetMapNameByID(29), -- Burning Steppes
				[182] = GetMapNameByID(182), -- Felwood
				[465] = GetMapNameByID(465), -- Hellfire Peninsula
				[261] = GetMapNameByID(261), -- Silithus
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[13463] = { -- Dreamfoil
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[181271] = { -- Dreaming Glory
			id = 181271,
			name = THN("Dreaming Glory"),
			expansion = 1,
			zones = {
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[465] = GetMapNameByID(465), -- Hellfire Peninsula
				[477] = GetMapNameByID(477), -- Nagrand
				[479] = GetMapNameByID(479), -- Netherstorm
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[481] = GetMapNameByID(481), -- Shattrath City
				[729] = GetMapNameByID(729), -- Tempest Keep: The Botanica
				[478] = GetMapNameByID(478), -- Terokkar Forest
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[22786] = { -- Dreaming Glory
					rate = 1,
					maxAmount = 8,
				},
				[22575] = { -- Mote of Life
					rate = 0.1462,
					maxAmount = 3,
				},
				[24401] = { -- Unidentified Plant Parts
					rate = 0.0359,
				},
				[22794] = { -- Fel Lotus
					rate = 0.0279,
					maxAmount = 4,
				},
				[35229] = { -- Nether Residue
					rate = 0.0108,
				},
			},
		}

		herbalismNodes[1619] = { -- Earthroot
			id = 1619,
			name = THN("Earthroot"),
			expansion = 0,
			zones = {
				[181] = GetMapNameByID(181), -- Azshara
				[464] = GetMapNameByID(464), -- Azuremyst Isle
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[27] = GetMapNameByID(27), -- Dun Morogh
				[4] = GetMapNameByID(4), -- Durotar
				[30] = GetMapNameByID(30), -- Elwynn Forest
				[462] = GetMapNameByID(462), -- Eversong Woods
				[463] = GetMapNameByID(463), -- Ghostlands
				[539] = GetMapNameByID(539), -- Gilneas
				[611] = GetMapNameByID(611), -- Gilneas City
				[35] = GetMapNameByID(35), -- Loch Modan
				[9] = GetMapNameByID(9), -- Mulgore
				[11] = GetMapNameByID(11), -- Northern Barrens
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[41] = GetMapNameByID(41), -- Teldrassil
				[544] = GetMapNameByID(544), -- The Lost Isles
				[808] = GetMapNameByID(808), -- The Wandering Isle
				[20] = GetMapNameByID(20), -- Tirisfal Glades
				[749] = GetMapNameByID(749), -- Wailing Caverns
				[39] = GetMapNameByID(39), -- Westfall
			},
			contains = {
				[2449] = { -- Earthroot
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[2042] = { -- Fadeleaf
			id = 2042,
			name = THN("Fadeleaf"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[121] = GetMapNameByID(121), -- Feralas
				[761] = GetMapNameByID(761), -- Razorfen Kraul
				[689] = GetMapNameByID(689), -- Stranglethorn Vale
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[26] = GetMapNameByID(26), -- The Hinterlands
				[22] = GetMapNameByID(22), -- Western Plaguelands
			},
			contains = {
				[3818] = { -- Fadeleaf
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[181270] = { -- Felweed
			id = 181270,
			name = THN("Felweed"),
			expansion = 1,
			zones = {
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[728] = GetMapNameByID(728), -- Coilfang Reservoir: The Slave Pens
				[726] = GetMapNameByID(726), -- Coilfang Reservoir: The Underbog
				[465] = GetMapNameByID(465), -- Hellfire Peninsula
				[477] = GetMapNameByID(477), -- Nagrand
				[479] = GetMapNameByID(479), -- Netherstorm
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[729] = GetMapNameByID(729), -- Tempest Keep: The Botanica
				[478] = GetMapNameByID(478), -- Terokkar Forest
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[22785] = { -- Felweed
					rate = 1,
					maxAmount = 8,
				},
				[22575] = { -- Mote of Life
					rate = 0.1471,
					maxAmount = 2,
				},
				[22795] = { -- Fel Blossom
					rate = 0.0989,
				},
				[24401] = { -- Unidentified Plant Parts
					rate = 0.0418,
				},
				[22794] = { -- Fel Lotus
					rate = 0.0277,
					maxAmount = 5,
				},
				[35229] = { -- Nether Residue
					rate = 0.0069,
				},
			},
		}

		herbalismNodes[2866] = { -- Firebloom
			id = 2866,
			name = THN("Firebloom"),
			expansion = 0,
			zones = {
				[17] = GetMapNameByID(17), -- Badlands
				[29] = GetMapNameByID(29), -- Burning Steppes
				[28] = GetMapNameByID(28), -- Searing Gorge
				[161] = GetMapNameByID(161), -- Tanaris
			},
			contains = {
				[4625] = { -- Firebloom
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[191303] = { -- Firethorn
			id = 191303,
			name = THN("Firethorn"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
			},
			contains = {
				[39970] = { -- Fire Leaf
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37704] = { -- Crystallized Life
					rate = 0.3084,
					maxAmount = 3,
				},
				[36908] = { -- Frost Lotus
					rate = 0.0772,
					maxAmount = 4,
				},
			},
		}

		herbalismNodes[228572] = { -- Fireweed
			id = 228572,
			name = THN("Fireweed"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[976] = GetMapNameByID(976), -- Frostwall
				[971] = GetMapNameByID(971), -- Lunarfall
				[948] = GetMapNameByID(948), -- Spires of Arak
			},
			contains = {
				[109125] = { -- Fireweed
					rate = 0.9566,
					maxAmount = 15,
				},
				[116053] = { -- Draenic Seeds
					rate = 0.8371,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.1008,
					maxAmount = 3,
				},
				[109625] = { -- Broken Fireweed Stem
					rate = 0.043,
					maxAmount = 9,
				},
				[111350] = { -- A Compendium of the Herbs of Draenor
					rate = 0.0071,
				},
			},
		}

		herbalismNodes[181276] = { -- Flame Cap
			id = 181276,
			name = THN("Flame Cap"),
			expansion = 1,
			zones = {
				[728] = GetMapNameByID(728), -- Coilfang Reservoir: The Slave Pens
				[727] = GetMapNameByID(727), -- Coilfang Reservoir: The Steamvault
				[726] = GetMapNameByID(726), -- Coilfang Reservoir: The Underbog
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[22788] = { -- Flame Cap
					rate = 1,
					maxAmount = 7,
				},
				[22794] = { -- Fel Lotus
					rate = 0.0273,
					maxAmount = 2,
				},
				[35229] = { -- Nether Residue
					rate = 0.0044,
				},
			},
		}

		herbalismNodes[209355] = { -- Fool's Cap
			id = 209355,
			name = THN("Fool's Cap"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[806] = GetMapNameByID(806), -- The Jade Forest
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[79011] = { -- Fool's Cap
					rate = 1,
					maxAmount = 9,
				},
				[89640] = { -- Life Spirit
					rate = 0.0973,
					minAmount = 2,
					maxAmount = 3,
				},
				[97623] = { -- Fool's Cap Spores
					rate = 0.0788,
					maxAmount = 13,
				},
				[89641] = { -- Water Spirit
					rate = 0.0726,
				},
				[72238] = { -- Golden Lotus
					rate = 0.0091,
					maxAmount = 4,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.004,
				},
			},
		}

		herbalismNodes[190176] = { -- Frost Lotus
			id = 190176,
			name = THN("Frost Lotus"),
			expansion = 2,
			zones = {
				[529] = GetMapNameByID(529), -- Ulduar
				[501] = GetMapNameByID(501), -- Wintergrasp
			},
			contains = {
				[36908] = { -- Frost Lotus
					rate = 1,
					maxAmount = 5,
				},
				[37921] = { -- Deadnettle
					rate = 0.3586,
					maxAmount = 5,
				},
				[37704] = { -- Crystallized Life
					rate = 0.3084,
					maxAmount = 3,
				},
			},
		}

		herbalismNodes[228571] = { -- Frostweed
			id = 228571,
			name = THN("Frostweed"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[976] = GetMapNameByID(976), -- Frostwall
				[971] = GetMapNameByID(971), -- Lunarfall
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
			},
			contains = {
				[109124] = { -- Frostweed
					rate = 0.9123,
					maxAmount = 15,
				},
				[116053] = { -- Draenic Seeds
					rate = 0.6047,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.0992,
					maxAmount = 3,
				},
				[109624] = { -- Broken Frostweed Stem
					rate = 0.0875,
					maxAmount = 9,
				},
				[111350] = { -- A Compendium of the Herbs of Draenor
					rate = 0.0174,
				},
			},
		}

		herbalismNodes[190173] = { -- Frozen Herb
			id = 190173,
			name = THN("Frozen Herb"),
			expansion = 0,
			zones = {
				[488] = GetMapNameByID(488), -- Dragonblight
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[520] = GetMapNameByID(520), -- The Nexus: The Nexus
				[501] = GetMapNameByID(501), -- Wintergrasp
				[496] = GetMapNameByID(496), -- Zul'Drak
			},
			contains = {
				[36901] = { -- Goldclover
					rate = 0.5367,
					maxAmount = 8,
				},
				[36907] = { -- Talandra's Rose
					rate = 0.1702,
					maxAmount = 8,
				},
				[36904] = { -- Tiger Lily
					rate = 0.1672,
					maxAmount = 8,
				},
				[2453] = { -- Bruiseweed
					rate = 0.0513,
					minAmount = 2,
					maxAmount = 7,
				},
				[2450] = { -- Briarthorn
					rate = 0.0504,
					minAmount = 2,
					maxAmount = 7,
				},
				[785] = { -- Mageroyal
					rate = 0.0244,
					minAmount = 2,
					maxAmount = 6,
				},
			},
		}

		herbalismNodes[142144] = { -- Ghost Mushroom
			id = 142144,
			name = THN("Ghost Mushroom"),
			expansion = 0,
			zones = {
				[699] = GetMapNameByID(699), -- Dire Maul
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[8845] = { -- Ghost Mushroom
					rate = 1,
					maxAmount = 6,
				},
				[108337] = { -- Ghost Mushroom Cap
					rate = 0.0316,
					minAmount = 2,
					maxAmount = 17,
				},
			},
		}

		herbalismNodes[189973] = { -- Goldclover
			id = 189973,
			name = THN("Goldclover"),
			expansion = 2,
			zones = {
				[533] = GetMapNameByID(533), -- Azjol-Nerub: Azjol-Nerub
				[486] = GetMapNameByID(486), -- Borean Tundra
				[488] = GetMapNameByID(488), -- Dragonblight
				[490] = GetMapNameByID(490), -- Grizzly Hills
				[491] = GetMapNameByID(491), -- Howling Fjord
				[493] = GetMapNameByID(493), -- Sholazar Basin
			},
			contains = {
				[36901] = { -- Goldclover
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37921] = { -- Deadnettle
					rate = 0.35,
					maxAmount = 8,
				},
				[37704] = { -- Crystallized Life
					rate = 0.3062,
					maxAmount = 3,
				},
				[36908] = { -- Frost Lotus
					rate = 0.0772,
					maxAmount = 5,
				},
			},
		}

		herbalismNodes[209354] = { -- Golden Lotus
			id = 209354,
			name = THN("Golden Lotus"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[806] = GetMapNameByID(806), -- The Jade Forest
				[873] = GetMapNameByID(873), -- The Veiled Stair
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
				[811] = GetMapNameByID(811), -- Vale of Eternal Blossoms
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[72238] = { -- Golden Lotus
					rate = 1,
					maxAmount = 5,
				},
				[89640] = { -- Life Spirit
					rate = 0.099,
					minAmount = 2,
					maxAmount = 3,
				},
				[89641] = { -- Water Spirit
					rate = 0.0746,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0003,
				},
			},
		}

		herbalismNodes[176583] = { -- Golden Sansam
			id = 176583,
			name = THN("Golden Sansam"),
			expansion = 0,
			zones = {
				[17] = GetMapNameByID(17), -- Badlands
				[19] = GetMapNameByID(19), -- Blasted Lands
				[182] = GetMapNameByID(182), -- Felwood
				[465] = GetMapNameByID(465), -- Hellfire Peninsula
				[479] = GetMapNameByID(479), -- Netherstorm
				[261] = GetMapNameByID(261), -- Silithus
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[13464] = { -- Golden Sansam
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[2046] = { -- Goldthorn
			id = 2046,
			name = THN("Goldthorn"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[101] = GetMapNameByID(101), -- Desolace
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[121] = GetMapNameByID(121), -- Feralas
				[760] = GetMapNameByID(760), -- Razorfen Downs
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[26] = GetMapNameByID(26), -- The Hinterlands
				[851] = GetMapNameByID(851), -- Theramore's Fall (H)
			},
			contains = {
				[3821] = { -- Goldthorn
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[228573] = { -- Gorgrond Flytrap
			id = 228573,
			name = THN("Gorgrond Flytrap"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[976] = GetMapNameByID(976), -- Frostwall
				[949] = GetMapNameByID(949), -- Gorgrond
				[971] = GetMapNameByID(971), -- Lunarfall
				[948] = GetMapNameByID(948), -- Spires of Arak
			},
			contains = {
				[109126] = { -- Gorgrond Flytrap
					rate = 0.9634,
					maxAmount = 4,
				},
				[116053] = { -- Draenic Seeds
					rate = 0.8404,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.0995,
					maxAmount = 3,
				},
				[109626] = { -- Gorgrond Flytrap Ichor
					rate = 0.0365,
					maxAmount = 7,
				},
			},
		}

		herbalismNodes[1628] = { -- Grave Moss
			id = 1628,
			name = THN("Grave Moss"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[34] = GetMapNameByID(34), -- Duskwood
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[760] = GetMapNameByID(760), -- Razorfen Downs
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[3369] = { -- Grave Moss
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[209349] = { -- Green Tea Leaf
			id = 209349,
			name = THN("Green Tea Leaf"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[871] = GetMapNameByID(871), -- Scarlet Halls
				[874] = GetMapNameByID(874), -- Scarlet Monastery
				[887] = GetMapNameByID(887), -- Siege of Niuzao Temple
				[867] = GetMapNameByID(867), -- Temple of the Jade Serpent
				[806] = GetMapNameByID(806), -- The Jade Forest
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
				[811] = GetMapNameByID(811), -- Vale of Eternal Blossoms
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[72234] = { -- Green Tea Leaf
					rate = 1,
					maxAmount = 9,
				},
				[89640] = { -- Life Spirit
					rate = 0.0977,
					minAmount = 2,
					maxAmount = 3,
				},
				[89641] = { -- Water Spirit
					rate = 0.0735,
				},
				[97619] = { -- Torn Green Tea Leaf
					rate = 0.0273,
					maxAmount = 13,
				},
				[72238] = { -- Golden Lotus
					rate = 0.0108,
					maxAmount = 5,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0005,
				},
			},
		}

		herbalismNodes[142145] = { -- Gromsblood
			id = 142145,
			name = THN("Gromsblood"),
			expansion = 0,
			zones = {
				[43] = GetMapNameByID(43), -- Ashenvale
				[19] = GetMapNameByID(19), -- Blasted Lands
				[101] = GetMapNameByID(101), -- Desolace
				[182] = GetMapNameByID(182), -- Felwood
			},
			contains = {
				[8846] = { -- Gromsblood
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[202750] = { -- Heartblossom
			id = 202750,
			name = THN("Heartblossom"),
			expansion = 3,
			zones = {
				[640] = GetMapNameByID(640), -- Deepholm
			},
			contains = {
				[52986] = { -- Heartblossom
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[52329] = { -- Volatile Life
					rate = 0.3794,
					maxAmount = 3,
				},
				[63122] = { -- Lifegiving Seed
					rate = 0.097,
				},
			},
		}

		herbalismNodes[176588] = { -- Icecap
			id = 176588,
			name = THN("Icecap"),
			expansion = 0,
			zones = {
				[281] = GetMapNameByID(281), -- Winterspring
			},
			contains = {
				[13467] = { -- Icecap
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[190172] = { -- Icethorn
			id = 190172,
			name = THN("Icethorn"),
			expansion = 2,
			zones = {
				[492] = GetMapNameByID(492), -- Icecrown
				[528] = GetMapNameByID(528), -- The Nexus: The Oculus
				[495] = GetMapNameByID(495), -- The Storm Peaks
				[524] = GetMapNameByID(524), -- Utgarde Keep: Utgarde Pinnacle
				[501] = GetMapNameByID(501), -- Wintergrasp
				[496] = GetMapNameByID(496), -- Zul'Drak
			},
			contains = {
				[36906] = { -- Icethorn
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37704] = { -- Crystallized Life
					rate = 0.307,
					maxAmount = 3,
				},
				[36908] = { -- Frost Lotus
					rate = 0.076,
					maxAmount = 4,
				},
			},
		}

		herbalismNodes[2043] = { -- Khadgar's Whisker
			id = 2043,
			name = THN("Khadgar's Whisker"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[101] = GetMapNameByID(101), -- Desolace
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[121] = GetMapNameByID(121), -- Feralas
				[607] = GetMapNameByID(607), -- Southern Barrens
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[26] = GetMapNameByID(26), -- The Hinterlands
				[22] = GetMapNameByID(22), -- Western Plaguelands
			},
			contains = {
				[3358] = { -- Khadgar's Whisker
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[1624] = { -- Kingsblood
			id = 1624,
			name = THN("Kingsblood"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[101] = GetMapNameByID(101), -- Desolace
				[34] = GetMapNameByID(34), -- Duskwood
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[121] = GetMapNameByID(121), -- Feralas
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[607] = GetMapNameByID(607), -- Southern Barrens
				[26] = GetMapNameByID(26), -- The Hinterlands
				[749] = GetMapNameByID(749), -- Wailing Caverns
				[22] = GetMapNameByID(22), -- Western Plaguelands
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[3356] = { -- Kingsblood
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[190171] = { -- Lichbloom
			id = 190171,
			name = THN("Lichbloom"),
			expansion = 2,
			zones = {
				[492] = GetMapNameByID(492), -- Icecrown
				[495] = GetMapNameByID(495), -- The Storm Peaks
				[524] = GetMapNameByID(524), -- Utgarde Keep: Utgarde Pinnacle
				[501] = GetMapNameByID(501), -- Wintergrasp
				[496] = GetMapNameByID(496), -- Zul'Drak
			},
			contains = {
				[36905] = { -- Lichbloom
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37704] = { -- Crystallized Life
					rate = 0.3107,
					maxAmount = 3,
				},
				[36908] = { -- Frost Lotus
					rate = 0.0759,
					maxAmount = 4,
				},
			},
		}

		herbalismNodes[2041] = { -- Liferoot
			id = 2041,
			name = THN("Liferoot"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[101] = GetMapNameByID(101), -- Desolace
				[34] = GetMapNameByID(34), -- Duskwood
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[479] = GetMapNameByID(479), -- Netherstorm
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[607] = GetMapNameByID(607), -- Southern Barrens
				[689] = GetMapNameByID(689), -- Stranglethorn Vale
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[26] = GetMapNameByID(26), -- The Hinterlands
				[61] = GetMapNameByID(61), -- Thousand Needles
				[22] = GetMapNameByID(22), -- Western Plaguelands
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[3357] = { -- Liferoot
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[1620] = { -- Mageroyal
			id = 1620,
			name = THN("Mageroyal"),
			expansion = 0,
			zones = {
				[43] = GetMapNameByID(43), -- Ashenvale
				[181] = GetMapNameByID(181), -- Azshara
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[34] = GetMapNameByID(34), -- Duskwood
				[463] = GetMapNameByID(463), -- Ghostlands
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[35] = GetMapNameByID(35), -- Loch Modan
				[11] = GetMapNameByID(11), -- Northern Barrens
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[607] = GetMapNameByID(607), -- Southern Barrens
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[41] = GetMapNameByID(41), -- Teldrassil
				[749] = GetMapNameByID(749), -- Wailing Caverns
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[785] = { -- Mageroyal
					rate = 1,
					maxAmount = 8,
				},
				[2452] = { -- Swiftthistle
					rate = 0.1989,
					maxAmount = 7,
				},
				[5056] = { -- Root Sample
					rate = 0.0114,
				},
			},
		}

		herbalismNodes[181281] = { -- Mana Thistle
			id = 181281,
			name = THN("Mana Thistle"),
			expansion = 1,
			zones = {
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[499] = GetMapNameByID(499), -- Isle of Quel'Danas
				[477] = GetMapNameByID(477), -- Nagrand
				[479] = GetMapNameByID(479), -- Netherstorm
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[478] = GetMapNameByID(478), -- Terokkar Forest
			},
			contains = {
				[22793] = { -- Mana Thistle
					rate = 1,
					maxAmount = 7,
				},
				[22575] = { -- Mote of Life
					rate = 0.1497,
					maxAmount = 3,
				},
				[35229] = { -- Nether Residue
					rate = 0.0287,
				},
				[22794] = { -- Fel Lotus
					rate = 0.0274,
					maxAmount = 3,
				},
			},
		}

		herbalismNodes[176586] = { -- Mountain Silversage
			id = 176586,
			name = THN("Mountain Silversage"),
			expansion = 0,
			zones = {
				[19] = GetMapNameByID(19), -- Blasted Lands
				[465] = GetMapNameByID(465), -- Hellfire Peninsula
				[261] = GetMapNameByID(261), -- Silithus
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[281] = GetMapNameByID(281), -- Winterspring
			},
			contains = {
				[13465] = { -- Mountain Silversage
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[228575] = { -- Nagrand Arrowbloom
			id = 228575,
			name = THN("Nagrand Arrowbloom"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[976] = GetMapNameByID(976), -- Frostwall
				[971] = GetMapNameByID(971), -- Lunarfall
				[950] = GetMapNameByID(950), -- Nagrand
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[116053] = { -- Draenic Seeds
					rate = 1,
					maxAmount = 3,
				},
				[109128] = { -- Nagrand Arrowbloom
					rate = 0.9648,
					maxAmount = 15,
				},
				[120945] = { -- Primal Spirit
					rate = 0.1001,
					maxAmount = 3,
				},
				[109628] = { -- Nagrand Arrowbloom Petal
					rate = 0.035,
					maxAmount = 9,
				},
			},
		}

		herbalismNodes[181279] = { -- Netherbloom
			id = 181279,
			name = THN("Netherbloom"),
			expansion = 1,
			zones = {
				[479] = GetMapNameByID(479), -- Netherstorm
				[729] = GetMapNameByID(729), -- Tempest Keep: The Botanica
			},
			contains = {
				[22791] = { -- Netherbloom
					rate = 1,
					maxAmount = 8,
				},
				[22576] = { -- Mote of Mana
					rate = 0.147,
					maxAmount = 3,
				},
				[22794] = { -- Fel Lotus
					rate = 0.03,
					maxAmount = 3,
				},
				[35229] = { -- Nether Residue
					rate = 0.0159,
				},
			},
		}

		herbalismNodes[185881] = { -- Netherdust Bush
			id = 185881,
			name = THN("Netherdust Bush"),
			expansion = 1,
			zones = {
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
			},
			contains = {
				[32468] = { -- Netherdust Pollen
					rate = 1,
					maxAmount = 3,
				},
				[22576] = { -- Mote of Mana
					rate = 0.1481,
					maxAmount = 3,
				},
				[22794] = { -- Fel Lotus
					rate = 0.0188,
					maxAmount = 3,
				},
				[32506] = { -- Netherwing Egg
					rate = 0.0107,
				},
			},
		}

		herbalismNodes[181280] = { -- Nightmare Vine
			id = 181280,
			name = THN("Nightmare Vine"),
			expansion = 1,
			zones = {
				[475] = GetMapNameByID(475), -- Blade's Edge Mountains
				[465] = GetMapNameByID(465), -- Hellfire Peninsula
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
			},
			contains = {
				[22792] = { -- Nightmare Vine
					rate = 1,
					maxAmount = 8,
				},
				[22794] = { -- Fel Lotus
					rate = 0.032,
					maxAmount = 4,
				},
				[35229] = { -- Nether Residue
					rate = 0.0148,
				},
			},
		}

		herbalismNodes[1618] = { -- Peacebloom
			id = 1618,
			name = THN("Peacebloom"),
			expansion = 0,
			zones = {
				[181] = GetMapNameByID(181), -- Azshara
				[464] = GetMapNameByID(464), -- Azuremyst Isle
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[27] = GetMapNameByID(27), -- Dun Morogh
				[4] = GetMapNameByID(4), -- Durotar
				[30] = GetMapNameByID(30), -- Elwynn Forest
				[462] = GetMapNameByID(462), -- Eversong Woods
				[463] = GetMapNameByID(463), -- Ghostlands
				[539] = GetMapNameByID(539), -- Gilneas
				[611] = GetMapNameByID(611), -- Gilneas City
				[35] = GetMapNameByID(35), -- Loch Modan
				[9] = GetMapNameByID(9), -- Mulgore
				[11] = GetMapNameByID(11), -- Northern Barrens
				[41] = GetMapNameByID(41), -- Teldrassil
				[544] = GetMapNameByID(544), -- The Lost Isles
				[808] = GetMapNameByID(808), -- The Wandering Isle
				[362] = GetMapNameByID(362), -- Thunder Bluff
				[20] = GetMapNameByID(20), -- Tirisfal Glades
				[39] = GetMapNameByID(39), -- Westfall
			},
			contains = {
				[2447] = { -- Peacebloom
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[142140] = { -- Purple Lotus
			id = 142140,
			name = THN("Purple Lotus"),
			expansion = 0,
			zones = {
				[182] = GetMapNameByID(182), -- Felwood
			},
			contains = {
				[8831] = { -- Purple Lotus
					rate = 1,
					maxAmount = 8,
				},
				[8153] = { -- Wildvine
					rate = 0.0993,
					maxAmount = 4,
				},
			},
		}

		herbalismNodes[181275] = { -- Ragveil
			id = 181275,
			name = THN("Ragveil"),
			expansion = 1,
			zones = {
				[728] = GetMapNameByID(728), -- Coilfang Reservoir: The Slave Pens
				[726] = GetMapNameByID(726), -- Coilfang Reservoir: The Underbog
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[22787] = { -- Ragveil
					rate = 1,
					maxAmount = 8,
				},
				[24401] = { -- Unidentified Plant Parts
					rate = 0.2418,
				},
				[22575] = { -- Mote of Life
					rate = 0.1489,
					maxAmount = 3,
				},
				[22794] = { -- Fel Lotus
					rate = 0.021,
					maxAmount = 4,
				},
				[35229] = { -- Nether Residue
					rate = 0.0047,
				},
			},
		}

		herbalismNodes[209353] = { -- Rain Poppy
			id = 209353,
			name = THN("Rain Poppy"),
			expansion = 4,
			zones = {
				[806] = GetMapNameByID(806), -- The Jade Forest
				[951] = GetMapNameByID(951), -- Timeless Isle
				[811] = GetMapNameByID(811), -- Vale of Eternal Blossoms
			},
			contains = {
				[72237] = { -- Rain Poppy
					rate = 0.9795,
					maxAmount = 9,
				},
				[89640] = { -- Life Spirit
					rate = 0.0979,
					minAmount = 2,
					maxAmount = 3,
				},
				[89641] = { -- Water Spirit
					rate = 0.0726,
				},
				[97620] = { -- Rain Poppy Petal
					rate = 0.0421,
					maxAmount = 12,
				},
				[72238] = { -- Golden Lotus
					rate = 0.0096,
					maxAmount = 4,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0044,
				},
			},
		}

		herbalismNodes[214510] = { -- Sha-Touched Herb
			id = 214510,
			name = THN("Sha-Touched Herb"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[806] = GetMapNameByID(806), -- The Jade Forest
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[89639] = { -- Desecrated Herb
					rate = 0.3277,
					maxAmount = 7,
				},
				[72234] = { -- Green Tea Leaf
					rate = 0.3255,
					maxAmount = 9,
				},
				[79010] = { -- Snow Lily
					rate = 0.1669,
					maxAmount = 9,
				},
				[79011] = { -- Fool's Cap
					rate = 0.1593,
					maxAmount = 9,
				},
				[89640] = { -- Life Spirit
					rate = 0.0191,
					minAmount = 2,
					maxAmount = 3,
				},
				[72238] = { -- Golden Lotus
					rate = 0.0175,
					maxAmount = 5,
				},
				[97624] = { -- Desecrated Herb Pod
					rate = 0.0173,
					maxAmount = 11,
				},
				[89641] = { -- Water Spirit
					rate = 0.0145,
				},
				[97622] = { -- Snow Lily Petal
					rate = 0.0135,
					maxAmount = 10,
				},
			},
		}

		herbalismNodes[209350] = { -- Silkweed
			id = 209350,
			name = THN("Silkweed"),
			expansion = 4,
			zones = {
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[806] = GetMapNameByID(806), -- The Jade Forest
				[873] = GetMapNameByID(873), -- The Veiled Stair
				[951] = GetMapNameByID(951), -- Timeless Isle
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[72235] = { -- Silkweed
					rate = 1,
					maxAmount = 9,
				},
				[89640] = { -- Life Spirit
					rate = 0.0982,
					minAmount = 2,
					maxAmount = 3,
				},
				[89641] = { -- Water Spirit
					rate = 0.0748,
				},
				[97621] = { -- Silkweed Stem
					rate = 0.0329,
					maxAmount = 13,
				},
				[72238] = { -- Golden Lotus
					rate = 0.0111,
					maxAmount = 5,
				},
				[103643] = { -- Dew of Eternal Morning
					rate = 0.0013,
				},
			},
		}

		herbalismNodes[1617] = { -- Silverleaf
			id = 1617,
			name = THN("Silverleaf"),
			expansion = 0,
			zones = {
				[181] = GetMapNameByID(181), -- Azshara
				[464] = GetMapNameByID(464), -- Azuremyst Isle
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[27] = GetMapNameByID(27), -- Dun Morogh
				[4] = GetMapNameByID(4), -- Durotar
				[30] = GetMapNameByID(30), -- Elwynn Forest
				[462] = GetMapNameByID(462), -- Eversong Woods
				[463] = GetMapNameByID(463), -- Ghostlands
				[539] = GetMapNameByID(539), -- Gilneas
				[611] = GetMapNameByID(611), -- Gilneas City
				[35] = GetMapNameByID(35), -- Loch Modan
				[9] = GetMapNameByID(9), -- Mulgore
				[11] = GetMapNameByID(11), -- Northern Barrens
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[41] = GetMapNameByID(41), -- Teldrassil
				[544] = GetMapNameByID(544), -- The Lost Isles
				[808] = GetMapNameByID(808), -- The Wandering Isle
				[362] = GetMapNameByID(362), -- Thunder Bluff
				[20] = GetMapNameByID(20), -- Tirisfal Glades
				[889] = GetMapNameByID(889), -- Valley of Trials
				[39] = GetMapNameByID(39), -- Westfall
			},
			contains = {
				[765] = { -- Silverleaf
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[209351] = { -- Snow Lily
			id = 209351,
			name = THN("Snow Lily"),
			expansion = 4,
			zones = {
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[877] = GetMapNameByID(877), -- Shado-Pan Monastery
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[79010] = { -- Snow Lily
					rate = 1,
					maxAmount = 9,
				},
				[89640] = { -- Life Spirit
					rate = 0.0999,
					minAmount = 2,
					maxAmount = 3,
				},
				[89641] = { -- Water Spirit
					rate = 0.0696,
				},
				[97622] = { -- Snow Lily Petal
					rate = 0.0486,
					maxAmount = 12,
				},
				[72238] = { -- Golden Lotus
					rate = 0.0089,
					maxAmount = 4,
				},
			},
		}

		herbalismNodes[176587] = { -- Sorrowmoss
			id = 176587,
			name = THN("Sorrowmoss"),
			expansion = 0,
			zones = {
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
			},
			contains = {
				[13466] = { -- Sorrowmoss
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[228574] = { -- Starflower
			id = 228574,
			name = THN("Starflower"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[976] = GetMapNameByID(976), -- Frostwall
				[971] = GetMapNameByID(971), -- Lunarfall
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
			},
			contains = {
				[109127] = { -- Starflower
					rate = 0.9485,
					maxAmount = 15,
				},
				[116053] = { -- Draenic Seeds
					rate = 0.7693,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.0985,
					maxAmount = 3,
				},
				[109627] = { -- Starflower Petal
					rate = 0.0511,
					maxAmount = 9,
				},
				[111350] = { -- A Compendium of the Herbs of Draenor
					rate = 0.007,
				},
			},
		}

		herbalismNodes[202748] = { -- Stormvine
			id = 202748,
			name = THN("Stormvine"),
			expansion = 3,
			zones = {
				[606] = GetMapNameByID(606), -- Mount Hyjal
				[614] = GetMapNameByID(614), -- Vashj'ir: Abyssal Depths
				[610] = GetMapNameByID(610), -- Vashj'ir: Kelp'thar Forest
				[615] = GetMapNameByID(615), -- Vashj'ir: Shimmering Expanse
			},
			contains = {
				[52984] = { -- Stormvine
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[52329] = { -- Volatile Life
					rate = 0.3882,
					maxAmount = 3,
				},
				[63122] = { -- Lifegiving Seed
					rate = 0.0987,
				},
			},
		}

		herbalismNodes[2045] = { -- Stranglekelp
			id = 2045,
			name = THN("Stranglekelp"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[688] = GetMapNameByID(688), -- Blackfathom Deeps
				[476] = GetMapNameByID(476), -- Bloodmyst Isle
				[42] = GetMapNameByID(42), -- Darkshore
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[463] = GetMapNameByID(463), -- Ghostlands
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[26] = GetMapNameByID(26), -- The Hinterlands
				[61] = GetMapNameByID(61), -- Thousand Needles
				[22] = GetMapNameByID(22), -- Western Plaguelands
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[3820] = { -- Stranglekelp
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[142142] = { -- Sungrass
			id = 142142,
			name = THN("Sungrass"),
			expansion = 0,
			zones = {
				[17] = GetMapNameByID(17), -- Badlands
				[29] = GetMapNameByID(29), -- Burning Steppes
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[28] = GetMapNameByID(28), -- Searing Gorge
				[261] = GetMapNameByID(261), -- Silithus
				[607] = GetMapNameByID(607), -- Southern Barrens
				[161] = GetMapNameByID(161), -- Tanaris
				[61] = GetMapNameByID(61), -- Thousand Needles
				[201] = GetMapNameByID(201), -- Un'Goro Crater
			},
			contains = {
				[8838] = { -- Sungrass
					rate = 1,
					maxAmount = 8,
				},
			},
		}

		herbalismNodes[228576] = { -- Talador Orchid
			id = 228576,
			name = THN("Talador Orchid"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
				[976] = GetMapNameByID(976), -- Frostwall
				[971] = GetMapNameByID(971), -- Lunarfall
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[109129] = { -- Talador Orchid
					rate = 0.9481,
					maxAmount = 15,
				},
				[116053] = { -- Draenic Seeds
					rate = 0.8204,
					maxAmount = 3,
				},
				[120945] = { -- Primal Spirit
					rate = 0.1007,
					maxAmount = 3,
				},
				[109629] = { -- Talador Orchid Petal
					rate = 0.0513,
					maxAmount = 9,
				},
			},
		}

		herbalismNodes[190170] = { -- Talandra's Rose
			id = 190170,
			name = THN("Talandra's Rose"),
			expansion = 2,
			zones = {
				[522] = GetMapNameByID(522), -- Azjol-Nerub: Ahn'kahet: The Old Kingdom
				[488] = GetMapNameByID(488), -- Dragonblight
				[534] = GetMapNameByID(534), -- Drak'Tharon Keep
				[490] = GetMapNameByID(490), -- Grizzly Hills
				[530] = GetMapNameByID(530), -- Gundrak
				[496] = GetMapNameByID(496), -- Zul'Drak
			},
			contains = {
				[36907] = { -- Talandra's Rose
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37921] = { -- Deadnettle
					rate = 0.3472,
					maxAmount = 8,
				},
				[37704] = { -- Crystallized Life
					rate = 0.3148,
					maxAmount = 3,
				},
				[36908] = { -- Frost Lotus
					rate = 0.0764,
					maxAmount = 4,
				},
			},
		}

		herbalismNodes[181277] = { -- Terocone
			id = 181277,
			name = THN("Terocone"),
			expansion = 1,
			zones = {
				[473] = GetMapNameByID(473), -- Shadowmoon Valley
				[729] = GetMapNameByID(729), -- Tempest Keep: The Botanica
				[478] = GetMapNameByID(478), -- Terokkar Forest
			},
			contains = {
				[22789] = { -- Terocone
					rate = 1,
					maxAmount = 8,
				},
				[22575] = { -- Mote of Life
					rate = 0.1477,
					maxAmount = 3,
				},
				[22794] = { -- Fel Lotus
					rate = 0.0288,
					maxAmount = 4,
				},
				[35229] = { -- Nether Residue
					rate = 0.0112,
				},
			},
		}

		herbalismNodes[190169] = { -- Tiger Lily
			id = 190169,
			name = THN("Tiger Lily"),
			expansion = 2,
			zones = {
				[533] = GetMapNameByID(533), -- Azjol-Nerub: Azjol-Nerub
				[486] = GetMapNameByID(486), -- Borean Tundra
				[490] = GetMapNameByID(490), -- Grizzly Hills
				[491] = GetMapNameByID(491), -- Howling Fjord
				[493] = GetMapNameByID(493), -- Sholazar Basin
			},
			contains = {
				[36904] = { -- Tiger Lily
					rate = 1,
					minAmount = 2,
					maxAmount = 8,
				},
				[37921] = { -- Deadnettle
					rate = 0.3512,
					maxAmount = 8,
				},
				[37704] = { -- Crystallized Life
					rate = 0.3082,
					maxAmount = 3,
				},
				[36908] = { -- Frost Lotus
					rate = 0.0764,
					maxAmount = 5,
				},
			},
		}

		herbalismNodes[202751] = { -- Twilight Jasmine
			id = 202751,
			name = THN("Twilight Jasmine"),
			expansion = 3,
			zones = {
				[700] = GetMapNameByID(700), -- Twilight Highlands
			},
			contains = {
				[52987] = { -- Twilight Jasmine
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[52329] = { -- Volatile Life
					rate = 0.3923,
					maxAmount = 3,
				},
				[63122] = { -- Lifegiving Seed
					rate = 0.0971,
				},
			},
		}

		herbalismNodes[202752] = { -- Whiptail
			id = 202752,
			name = THN("Whiptail"),
			expansion = 3,
			zones = {
				[708] = GetMapNameByID(708), -- Tol Barad
				[720] = GetMapNameByID(720), -- Uldum
			},
			contains = {
				[52988] = { -- Whiptail
					rate = 1,
					minAmount = 2,
					maxAmount = 9,
				},
				[52329] = { -- Volatile Life
					rate = 0.3922,
					maxAmount = 3,
				},
				[63122] = { -- Lifegiving Seed
					rate = 0.1001,
				},
			},
		}

		herbalismNodes[1623] = { -- Wild Steelbloom
			id = 1623,
			name = THN("Wild Steelbloom"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[101] = GetMapNameByID(101), -- Desolace
				[34] = GetMapNameByID(34), -- Duskwood
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[689] = GetMapNameByID(689), -- Stranglethorn Vale
				[26] = GetMapNameByID(26), -- The Hinterlands
				[700] = GetMapNameByID(700), -- Twilight Highlands
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[3355] = { -- Wild Steelbloom
					rate = 1,
					maxAmount = 8,
				},
			},
		}

	end
	
	-- FISHING
	local fishingNodes = {}

	do -- node data
		fishingNodes[229072] = { -- Abyssal Gulper School
			id = 229072,
			name = TFN("Abyssal Gulper School"),
			expansion = 5,
			zones = {
				[948] = GetMapNameByID(948), -- Spires of Arak
			},
			contains = {
				[116751] = { -- Abyssal Gulper Lunker
					rate = -1,
				},
				[111659] = { -- Small Abyssal Gulper Eel
					rate = -1,
				},
				[111664] = { -- Abyssal Gulper Eel
					rate = -1,
				},
				[111671] = { -- Enormous Abyssal Gulper Eel
					rate = -1,
				},
			},
		}

		fishingNodes[202778] = { -- Albino Cavefish School
			id = 202778,
			name = TFN("Albino Cavefish School"),
			expansion = 3,
			zones = {
				[640] = GetMapNameByID(640), -- Deepholm
			},
			contains = {
				[53065] = { -- Albino Cavefish
					rate = 1,
				},
				[52326] = { -- Volatile Water
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[22739] = { -- Tome of Polymorph: Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[202779] = { -- Blackbelly Mudfish School
			id = 202779,
			name = TFN("Blackbelly Mudfish School"),
			expansion = 3,
			zones = {
				[720] = GetMapNameByID(720), -- Uldum
			},
			contains = {
				[53066] = { -- Blackbelly Mudfish
					rate = 1,
				},
				[52326] = { -- Volatile Water
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[22739] = { -- Tome of Polymorph: Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[229073] = { -- Blackwater Whiptail School
			id = 229073,
			name = TFN("Blackwater Whiptail School"),
			expansion = 5,
			zones = {
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[111663] = { -- Blackwater Whiptail
					rate = 1,
				},
				[111662] = { -- Small Blackwater Whiptail
					rate = -1,
				},
				[111670] = { -- Enormous Blackwater Whiptail
					rate = -1,
				},
				[116749] = { -- Blackwater Whiptail Lunker
					rate = -1,
				},
			},
		}

		fishingNodes[229069] = { -- Blind Lake Sturgeon School
			id = 229069,
			name = TFN("Blind Lake Sturgeon School"),
			expansion = 5,
			zones = {
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
			},
			contains = {
				[111667] = { -- Blind Lake Sturgeon
					rate = 1,
				},
				[111674] = { -- Enormous Blind Lake Sturgeon
					rate = -1,
				},
				[116750] = { -- Blind Lake Sturgeon Lunker
					rate = -1,
				},
				[111652] = { -- Small Blind Lake Sturgeon
					rate = -1,
				},
			},
		}

		fishingNodes[180901] = { -- Bloodsail Wreckage
			id = 180901,
			name = TFN("Bloodsail Wreckage"),
			expansion = 0,
			zones = {
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
			},
			contains = {
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21150] = { -- Iron Bound Trunk
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
			},
		}

		fishingNodes[182959] = { -- Bluefish School
			id = 182959,
			name = TFN("Bluefish School"),
			expansion = 1,
			zones = {
				[477] = GetMapNameByID(477), -- Nagrand
			},
			contains = {
				[27511] = { -- Inscribed Scrollcase
					rate = -1,
				},
				[27437] = { -- Icefin Bluefish
					rate = -1,
				},
				[22578] = { -- Mote of Water
					rate = -1,
				},
				[27442] = { -- Goldenscale Vendorfish
					rate = -1,
				},
				[27513] = { -- Curious Crate
					rate = -1,
				},
			},
		}

		fishingNodes[192051] = { -- Borean Man O' War School
			id = 192051,
			name = TFN("Borean Man O' War School"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
			},
			contains = {
				[41805] = { -- Borean Man O' War
					rate = 1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
			},
		}

		fishingNodes[182954] = { -- Brackish Mixed School
			id = 182954,
			name = TFN("Brackish Mixed School"),
			expansion = 1,
			zones = {
				[478] = GetMapNameByID(478), -- Terokkar Forest
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[27511] = { -- Inscribed Scrollcase
					rate = -1,
				},
				[27438] = { -- Golden Darter
					rate = -1,
				},
				[27425] = { -- Spotted Feltail
					rate = -1,
				},
				[22578] = { -- Mote of Water
					rate = -1,
				},
				[27442] = { -- Goldenscale Vendorfish
					rate = -1,
				},
				[27513] = { -- Curious Crate
					rate = -1,
				},
			},
		}

		fishingNodes[218633] = { -- Brew Frenzied Emperor Salmon
			id = 218633,
			name = TFN("Brew Frenzied Emperor Salmon"),
			expansion = 4,
			zones = {
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[74859] = { -- Emperor Salmon
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[218648] = { -- Crowded Redbelly Mandarin
			id = 218648,
			name = TFN("Crowded Redbelly Mandarin"),
			expansion = 4,
			zones = {
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[74860] = { -- Redbelly Mandarin
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[94933] = { -- Tiny Blue Carp
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
			},
		}

		fishingNodes[192053] = { -- Deep Sea Monsterbelly School
			id = 192053,
			name = TFN("Deep Sea Monsterbelly School"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
				[488] = GetMapNameByID(488), -- Dragonblight
				[491] = GetMapNameByID(491), -- Howling Fjord
			},
			contains = {
				[41800] = { -- Deep Sea Monsterbelly
					rate = 1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
			},
		}

		fishingNodes[208311] = { -- Deepsea Sagefish School
			id = 208311,
			name = TFN("Deepsea Sagefish School"),
			expansion = 3,
			zones = {
				[700] = GetMapNameByID(700), -- Twilight Highlands
			},
			contains = {
				[53072] = { -- Deepsea Sagefish
					rate = 1,
				},
				[52985] = { -- Azshara's Veil
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[22739] = { -- Tome of Polymorph: Turtle
					rate = -1,
				},
				[52326] = { -- Volatile Water
					rate = -1,
				},
			},
		}

		fishingNodes[192048] = { -- Dragonfin Angelfish School
			id = 192048,
			name = TFN("Dragonfin Angelfish School"),
			expansion = 2,
			zones = {
				[488] = GetMapNameByID(488), -- Dragonblight
			},
			contains = {
				[41807] = { -- Dragonfin Angelfish
					rate = 1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
			},
		}

		fishingNodes[212163] = { -- Emperor Salmon School
			id = 212163,
			name = TFN("Emperor Salmon School"),
			expansion = 4,
			zones = {
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[810] = GetMapNameByID(810), -- Townlong Steppes
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[74859] = { -- Emperor Salmon
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[192049] = { -- Fangtooth Herring School
			id = 192049,
			name = TFN("Fangtooth Herring School"),
			expansion = 2,
			zones = {
				[491] = GetMapNameByID(491), -- Howling Fjord
			},
			contains = {
				[41810] = { -- Fangtooth Herring
					rate = 1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
			},
		}

		fishingNodes[229068] = { -- Fat Sleeper School
			id = 229068,
			name = TFN("Fat Sleeper School"),
			expansion = 5,
			zones = {
				[950] = GetMapNameByID(950), -- Nagrand
			},
			contains = {
				[111668] = { -- Fat Sleeper
					rate = 1,
				},
				[111651] = { -- Small Fat Sleeper
					rate = -1,
				},
				[116753] = { -- Fat Sleeper Lunker
					rate = -1,
				},
				[111675] = { -- Enormous Fat Sleeper
					rate = -1,
				},
			},
		}

		fishingNodes[202780] = { -- Fathom Eel Swarm
			id = 202780,
			name = TFN("Fathom Eel Swarm"),
			expansion = 3,
			zones = {
				[709] = GetMapNameByID(709), -- Tol Barad Peninsula
				[720] = GetMapNameByID(720), -- Uldum
			},
			contains = {
				[53070] = { -- Fathom Eel
					rate = 1,
				},
				[52326] = { -- Volatile Water
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[22739] = { -- Tome of Polymorph: Turtle
					rate = -1,
				},
				[52985] = { -- Azshara's Veil
					rate = -1,
				},
			},
		}

		fishingNodes[229070] = { -- Fire Ammonite School
			id = 229070,
			name = TFN("Fire Ammonite School"),
			expansion = 5,
			zones = {
				[941] = GetMapNameByID(941), -- Frostfire Ridge
			},
			contains = {
				[116748] = { -- Fire Ammonite Lunker
					rate = -1,
				},
				[111656] = { -- Small Fire Ammonite
					rate = -1,
				},
				[111673] = { -- Enormous Fire Ammonite
					rate = -1,
				},
				[110291] = { -- Fire Ammonite Bait
					rate = -1,
				},
			},
		}

		fishingNodes[180683] = { -- Firefin Snapper School
			id = 180683,
			name = TFN("Firefin Snapper School"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[181] = GetMapNameByID(181), -- Azshara
				[19] = GetMapNameByID(19), -- Blasted Lands
				[42] = GetMapNameByID(42), -- Darkshore
				[101] = GetMapNameByID(101), -- Desolace
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[121] = GetMapNameByID(121), -- Feralas
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[11] = GetMapNameByID(11), -- Northern Barrens
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[607] = GetMapNameByID(607), -- Southern Barrens
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[161] = GetMapNameByID(161), -- Tanaris
				[26] = GetMapNameByID(26), -- The Hinterlands
				[61] = GetMapNameByID(61), -- Thousand Needles
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[6359] = { -- Firefin Snapper
					rate = 1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21150] = { -- Iron Bound Trunk
					rate = -1,
				},
			},
		}

		fishingNodes[180655] = { -- Floating Debris
			id = 180655,
			name = TFN("Floating Debris"),
			expansion = 0,
			zones = {
				[181] = GetMapNameByID(181), -- Azshara
				[42] = GetMapNameByID(42), -- Darkshore
				[34] = GetMapNameByID(34), -- Duskwood
				[11] = GetMapNameByID(11), -- Northern Barrens
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[39] = GetMapNameByID(39), -- Westfall
			},
			contains = {
				[20708] = { -- Tightly Sealed Trunk
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[3820] = { -- Stranglekelp
					rate = -1,
				},
			},
		}

		fishingNodes[180751] = { -- Floating Wreckage
			id = 180751,
			name = TFN("Floating Wreckage"),
			expansion = 0,
			zones = {
				[19] = GetMapNameByID(19), -- Blasted Lands
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[161] = GetMapNameByID(161), -- Tanaris
				[61] = GetMapNameByID(61), -- Thousand Needles
			},
			contains = {
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21228] = { -- Mithril Bound Trunk
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
			},
		}

		fishingNodes[212169] = { -- Giant Mantis Shrimp Swarm
			id = 212169,
			name = TFN("Giant Mantis Shrimp Swarm"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[929] = GetMapNameByID(929), -- Isle of Giants
				[928] = GetMapNameByID(928), -- Isle of Thunder
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[806] = GetMapNameByID(806), -- The Jade Forest
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[74857] = { -- Giant Mantis Shrimp
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
				[86545] = { -- Mimic Octopus
					rate = -1,
				},
			},
		}

		fishingNodes[192050] = { -- Glacial Salmon School
			id = 192050,
			name = TFN("Glacial Salmon School"),
			expansion = 2,
			zones = {
				[490] = GetMapNameByID(490), -- Grizzly Hills
			},
			contains = {
				[41809] = { -- Glacial Salmon
					rate = 1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
			},
		}

		fishingNodes[192059] = { -- Glassfin Minnow School
			id = 192059,
			name = TFN("Glassfin Minnow School"),
			expansion = 2,
			zones = {
				[510] = GetMapNameByID(510), -- Crystalsong Forest
			},
			contains = {
				[41814] = { -- Glassfin Minnow
					rate = 1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
			},
		}

		fishingNodes[218652] = { -- Glimmering Jewel Danio Pool
			id = 218652,
			name = TFN("Glimmering Jewel Danio Pool"),
			expansion = 4,
			zones = {
				[951] = GetMapNameByID(951), -- Timeless Isle
			},
			contains = {
				[74863] = { -- Jewel Danio
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[94933] = { -- Tiny Blue Carp
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[218629] = { -- Glowing Jade Lungfish
			id = 218629,
			name = TFN("Glowing Jade Lungfish"),
			expansion = 4,
			zones = {
				[806] = GetMapNameByID(806), -- The Jade Forest
			},
			contains = {
				[74856] = { -- Jade Lungfish
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[180684] = { -- Greater Sagefish School
			id = 180684,
			name = TFN("Greater Sagefish School"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[19] = GetMapNameByID(19), -- Blasted Lands
				[101] = GetMapNameByID(101), -- Desolace
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[23] = GetMapNameByID(23), -- Eastern Plaguelands
				[121] = GetMapNameByID(121), -- Feralas
				[241] = GetMapNameByID(241), -- Moonglade
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[26] = GetMapNameByID(26), -- The Hinterlands
				[61] = GetMapNameByID(61), -- Thousand Needles
				[201] = GetMapNameByID(201), -- Un'Goro Crater
				[22] = GetMapNameByID(22), -- Western Plaguelands
			},
			contains = {
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[21150] = { -- Iron Bound Trunk
					rate = -1,
				},
				[21153] = { -- Raw Greater Sagefish
					rate = -1,
				},
			},
		}

		fishingNodes[202777] = { -- Highland Guppy School
			id = 202777,
			name = TFN("Highland Guppy School"),
			expansion = 3,
			zones = {
				[700] = GetMapNameByID(700), -- Twilight Highlands
			},
			contains = {
				[53064] = { -- Highland Guppy
					rate = 1,
				},
				[52326] = { -- Volatile Water
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[22739] = { -- Tome of Polymorph: Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[182957] = { -- Highland Mixed School
			id = 182957,
			name = TFN("Highland Mixed School"),
			expansion = 1,
			zones = {
				[478] = GetMapNameByID(478), -- Terokkar Forest
			},
			contains = {
				[27511] = { -- Inscribed Scrollcase
					rate = -1,
				},
				[27438] = { -- Golden Darter
					rate = -1,
				},
				[27439] = { -- Furious Crawdad
					rate = -1,
				},
				[27388] = { -- Mr. Pinchy
					rate = -1,
				},
				[27442] = { -- Goldenscale Vendorfish
					rate = -1,
				},
				[22578] = { -- Mote of Water
					rate = -1,
				},
				[27513] = { -- Curious Crate
					rate = -1,
				},
			},
		}

		fishingNodes[192052] = { -- Imperial Manta Ray School
			id = 192052,
			name = TFN("Imperial Manta Ray School"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
				[488] = GetMapNameByID(488), -- Dragonblight
				[490] = GetMapNameByID(490), -- Grizzly Hills
				[491] = GetMapNameByID(491), -- Howling Fjord
			},
			contains = {
				[41802] = { -- Imperial Manta Ray
					rate = 1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
			},
		}

		fishingNodes[212171] = { -- Jade Lungfish School
			id = 212171,
			name = TFN("Jade Lungfish School"),
			expansion = 4,
			zones = {
				[806] = GetMapNameByID(806), -- The Jade Forest
			},
			contains = {
				[74856] = { -- Jade Lungfish
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[229067] = { -- Jawless Skulker School
			id = 229067,
			name = TFN("Jawless Skulker School"),
			expansion = 5,
			zones = {
				[949] = GetMapNameByID(949), -- Gorgrond
			},
			contains = {
				[111669] = { -- Jawless Skulker
					rate = 1,
				},
				[111650] = { -- Small Jawless Skulker
					rate = -1,
				},
				[111676] = { -- Enormous Jawless Skulker
					rate = -1,
				},
				[116752] = { -- Jawless Skulker Lunker
					rate = -1,
				},
			},
		}

		fishingNodes[221548] = { -- Jewel Danio School
			id = 221548,
			name = TFN("Jewel Danio School"),
			expansion = 4,
			zones = {
				[951] = GetMapNameByID(951), -- Timeless Isle
			},
			contains = {
				[74863] = { -- Jewel Danio
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[94933] = { -- Tiny Blue Carp
					rate = -1,
				},
				[104034] = { -- Purse of Timeless Coins
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[212172] = { -- Krasarang Paddlefish School
			id = 212172,
			name = TFN("Krasarang Paddlefish School"),
			expansion = 4,
			zones = {
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[74865] = { -- Krasarang Paddlefish
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[226967] = { -- Lagoon Pool
			id = 226967,
			name = TFN("Lagoon Pool"),
			expansion = 5,
			zones = {
				[941] = GetMapNameByID(941), -- Frostfire Ridge
			},
			contains = {
				[108738] = { -- Giant Draenor Clam
					rate = -1,
				},
			},
		}

		fishingNodes[218634] = { -- Large Pool of Brew Frenzied Emperor Salmon
			id = 218634,
			name = TFN("Large Pool of Brew Frenzied Emperor Salmon"),
			expansion = 4,
			zones = {
				[807] = GetMapNameByID(807), -- Valley of the Four Winds
			},
			contains = {
				[74859] = { -- Emperor Salmon
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[218649] = { -- Large Pool of Crowded Redbelly Mandarin
			id = 218649,
			name = TFN("Large Pool of Crowded Redbelly Mandarin"),
			expansion = 4,
			zones = {
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[74860] = { -- Redbelly Mandarin
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[94933] = { -- Tiny Blue Carp
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
			},
		}

		fishingNodes[218653] = { -- Large Pool of Glimmering Jewel Danio
			id = 218653,
			name = TFN("Large Pool of Glimmering Jewel Danio"),
			expansion = 4,
			zones = {
				[951] = GetMapNameByID(951), -- Timeless Isle
			},
			contains = {
				[74863] = { -- Jewel Danio
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[94933] = { -- Tiny Blue Carp
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[218630] = { -- Large Pool of Glowing Jade Lungfish
			id = 218630,
			name = TFN("Large Pool of Glowing Jade Lungfish"),
			expansion = 4,
			zones = {
				[806] = GetMapNameByID(806), -- The Jade Forest
			},
			contains = {
				[74856] = { -- Jade Lungfish
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[218651] = { -- Large Pool of Sha-Touched Spinefish
			id = 218651,
			name = TFN("Large Pool of Sha-Touched Spinefish"),
			expansion = 4,
			zones = {
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
			},
			contains = {
				[83064] = { -- Spinefish
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[94932] = { -- Tiny Red Carp
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[86544] = { -- Spinefish Alpha
					rate = -1,
				},
			},
		}

		fishingNodes[218636] = { -- Large Pool of Tiger Gourami Slush
			id = 218636,
			name = TFN("Large Pool of Tiger Gourami Slush"),
			expansion = 4,
			zones = {
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
			},
			contains = {
				[74861] = { -- Tiger Gourami
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
			},
		}

		fishingNodes[218539] = { -- Large Swarm of Migrated Reef Octopus
			id = 218539,
			name = TFN("Large Swarm of Migrated Reef Octopus"),
			expansion = 4,
			zones = {
				[806] = GetMapNameByID(806), -- The Jade Forest
			},
			contains = {
				[74864] = { -- Reef Octopus
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[86545] = { -- Mimic Octopus
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
			},
		}

		fishingNodes[218632] = { -- Large Swarm of Panicked Paddlefish
			id = 218632,
			name = TFN("Large Swarm of Panicked Paddlefish"),
			expansion = 4,
			zones = {
				[857] = GetMapNameByID(857), -- Krasarang Wilds
			},
			contains = {
				[74865] = { -- Krasarang Paddlefish
					rate = -1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[218576] = { -- Large Tangled Mantis Shrimp Cluster
			id = 218576,
			name = TFN("Large Tangled Mantis Shrimp Cluster"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
			},
			contains = {
				[86545] = { -- Mimic Octopus
					rate = -1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[74857] = { -- Giant Mantis Shrimp
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
			},
		}

		fishingNodes[216761] = { -- Mixed Ocean School
			id = 216761,
			name = TFN("Mixed Ocean School"),
			expansion = 0,
			zones = {
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
			},
			contains = {
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[6359] = { -- Firefin Snapper
					rate = -1,
				},
				[21150] = { -- Iron Bound Trunk
					rate = -1,
				},
				[6358] = { -- Oily Blackmouth
					rate = -1,
				},
			},
		}

		fishingNodes[192054] = { -- Moonglow Cuttlefish School
			id = 192054,
			name = TFN("Moonglow Cuttlefish School"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
				[488] = GetMapNameByID(488), -- Dragonblight
				[491] = GetMapNameByID(491), -- Howling Fjord
			},
			contains = {
				[41801] = { -- Moonglow Cuttlefish
					rate = 1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
			},
		}

		fishingNodes[202776] = { -- Mountain Trout School
			id = 202776,
			name = TFN("Mountain Trout School"),
			expansion = 3,
			zones = {
				[606] = GetMapNameByID(606), -- Mount Hyjal
			},
			contains = {
				[53063] = { -- Mountain Trout
					rate = 1,
				},
				[52326] = { -- Volatile Water
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[22739] = { -- Tome of Polymorph: Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[182958] = { -- Mudfish School
			id = 182958,
			name = TFN("Mudfish School"),
			expansion = 1,
			zones = {
				[477] = GetMapNameByID(477), -- Nagrand
			},
			contains = {
				[27511] = { -- Inscribed Scrollcase
					rate = -1,
				},
				[27513] = { -- Curious Crate
					rate = -1,
				},
				[22578] = { -- Mote of Water
					rate = -1,
				},
				[27442] = { -- Goldenscale Vendorfish
					rate = -1,
				},
				[27435] = { -- Figluster's Mudfish
					rate = -1,
				},
			},
		}

		fishingNodes[192046] = { -- Musselback Sculpin School
			id = 192046,
			name = TFN("Musselback Sculpin School"),
			expansion = 2,
			zones = {
				[486] = GetMapNameByID(486), -- Borean Tundra
			},
			contains = {
				[41806] = { -- Musselback Sculpin
					rate = 1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
			},
		}

		fishingNodes[192057] = { -- Nettlefish School
			id = 192057,
			name = TFN("Nettlefish School"),
			expansion = 2,
			zones = {
				[493] = GetMapNameByID(493), -- Sholazar Basin
			},
			contains = {
				[41813] = { -- Nettlefish
					rate = 1,
				},
				[37705] = { -- Crystallized Water
					rate = -1,
				},
				[44475] = { -- Reinforced Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[40199] = { -- Pygmy Suckerfish
					rate = -1,
				},
				[45902] = { -- Phantom Ghostfish
					rate = -1,
				},
			},
		}

		fishingNodes[236756] = { -- Oily Abyssal Gulper School
			id = 236756,
			name = TFN("Oily Abyssal Gulper School"),
			expansion = 5,
			zones = {
				[948] = GetMapNameByID(948), -- Spires of Arak
			},
			contains = {
				[111671] = { -- Enormous Abyssal Gulper Eel
					rate = -1,
				},
				[118046] = { -- Rubber Duck
					rate = -1,
				},
				[111659] = { -- Small Abyssal Gulper Eel
					rate = -1,
				},
				[116751] = { -- Abyssal Gulper Lunker
					rate = -1,
				},
				[111664] = { -- Abyssal Gulper Eel
					rate = -1,
				},
			},
		}

		fishingNodes[180682] = { -- Oily Blackmouth School
			id = 180682,
			name = TFN("Oily Blackmouth School"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[181] = GetMapNameByID(181), -- Azshara
				[19] = GetMapNameByID(19), -- Blasted Lands
				[42] = GetMapNameByID(42), -- Darkshore
				[101] = GetMapNameByID(101), -- Desolace
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[121] = GetMapNameByID(121), -- Feralas
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[11] = GetMapNameByID(11), -- Northern Barrens
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[607] = GetMapNameByID(607), -- Southern Barrens
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[161] = GetMapNameByID(161), -- Tanaris
				[26] = GetMapNameByID(26), -- The Hinterlands
				[61] = GetMapNameByID(61), -- Thousand Needles
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[6358] = { -- Oily Blackmouth
					rate = 1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[21150] = { -- Iron Bound Trunk
					rate = -1,
				},
				[3820] = { -- Stranglekelp
					rate = -1,
				},
			},
		}

		fishingNodes[237295] = { -- Oily Sea Scorpion School
			id = 237295,
			name = TFN("Oily Sea Scorpion School"),
			expansion = 5,
			zones = {
				[948] = GetMapNameByID(948), -- Spires of Arak
			},
			contains = {
				[111665] = { -- Sea Scorpion
					rate = 1,
				},
				[111672] = { -- Enormous Sea Scorpion
					rate = -1,
				},
				[111658] = { -- Small Sea Scorpion
					rate = -1,
				},
				[118046] = { -- Rubber Duck
					rate = -1,
				},
			},
		}

		fishingNodes[207734] = { -- Pool of Fire
			id = 207734,
			name = TFN("Pool of Fire"),
			expansion = 3,
			zones = {
				[606] = GetMapNameByID(606), -- Mount Hyjal
				[700] = GetMapNameByID(700), -- Twilight Highlands
			},
			contains = {
				[52325] = { -- Volatile Fire
					rate = -1,
				},
			},
		}

		fishingNodes[182951] = { -- Pure Water
			id = 182951,
			name = TFN("Pure Water"),
			expansion = 1,
			zones = {
				[477] = GetMapNameByID(477), -- Nagrand
			},
			contains = {
				[22578] = { -- Mote of Water
					rate = -1,
				},
			},
		}

		fishingNodes[221549] = { -- Redbelly Mandarin School
			id = 221549,
			name = TFN("Redbelly Mandarin School"),
			expansion = 4,
			zones = {
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[74860] = { -- Redbelly Mandarin
					rate = 1,
				},
				[94933] = { -- Tiny Blue Carp
					rate = -1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
			},
		}

		fishingNodes[212174] = { -- Reef Octopus Swarm
			id = 212174,
			name = TFN("Reef Octopus Swarm"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[929] = GetMapNameByID(929), -- Isle of Giants
				[928] = GetMapNameByID(928), -- Isle of Thunder
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[806] = GetMapNameByID(806), -- The Jade Forest
				[951] = GetMapNameByID(951), -- Timeless Isle
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[74864] = { -- Reef Octopus
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[86545] = { -- Mimic Octopus
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
			},
		}

		fishingNodes[216764] = { -- Sagefish School
			id = 216764,
			name = TFN("Sagefish School"),
			expansion = 0,
			zones = {
				[43] = GetMapNameByID(43), -- Ashenvale
				[181] = GetMapNameByID(181), -- Azshara
				[42] = GetMapNameByID(42), -- Darkshore
				[34] = GetMapNameByID(34), -- Duskwood
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[35] = GetMapNameByID(35), -- Loch Modan
				[11] = GetMapNameByID(11), -- Northern Barrens
				[37] = GetMapNameByID(37), -- Northern Stranglethorn
				[36] = GetMapNameByID(36), -- Redridge Mountains
				[21] = GetMapNameByID(21), -- Silverpine Forest
				[673] = GetMapNameByID(673), -- The Cape of Stranglethorn
				[39] = GetMapNameByID(39), -- Westfall
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[21071] = { -- Raw Sagefish
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[21113] = { -- Watertight Trunk
					rate = -1,
				},
				[3820] = { -- Stranglekelp
					rate = -1,
				},
			},
		}

		fishingNodes[237342] = { -- Savage Piranha Pool
			id = 237342,
			name = TFN("Savage Piranha Pool"),
			expansion = 5,
			zones = {
				[978] = GetMapNameByID(978), -- Ashran
			},
			contains = {
				[118565] = { -- Savage Piranha
					rate = 1,
				},
				[118564] = { -- Small Savage Piranha
					rate = -1,
				},
				[118566] = { -- Enormous Savage Piranha
					rate = -1,
				},
			},
		}

		fishingNodes[182956] = { -- School of Darter
			id = 182956,
			name = TFN("School of Darter"),
			expansion = 1,
			zones = {
				[478] = GetMapNameByID(478), -- Terokkar Forest
			},
			contains = {
				[27511] = { -- Inscribed Scrollcase
					rate = -1,
				},
				[27438] = { -- Golden Darter
					rate = -1,
				},
				[22578] = { -- Mote of Water
					rate = -1,
				},
				[27442] = { -- Goldenscale Vendorfish
					rate = -1,
				},
				[27513] = { -- Curious Crate
					rate = -1,
				},
			},
		}

		fishingNodes[180658] = { -- School of Deviate Fish
			id = 180658,
			name = TFN("School of Deviate Fish"),
			expansion = 0,
			zones = {
				[11] = GetMapNameByID(11), -- Northern Barrens
			},
			contains = {
				[6522] = { -- Deviate Fish
					rate = 1,
				},
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[20708] = { -- Tightly Sealed Trunk
					rate = -1,
				},
			},
		}

		fishingNodes[180662] = { -- Schooner Wreckage
			id = 180662,
			name = TFN("Schooner Wreckage"),
			expansion = 0,
			zones = {
				[16] = GetMapNameByID(16), -- Arathi Highlands
				[43] = GetMapNameByID(43), -- Ashenvale
				[24] = GetMapNameByID(24), -- Hillsbrad Foothills
				[81] = GetMapNameByID(81), -- Stonetalon Mountains
				[40] = GetMapNameByID(40), -- Wetlands
			},
			contains = {
				[21113] = { -- Watertight Trunk
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[3820] = { -- Stranglekelp
					rate = -1,
				},
			},
		}

		fishingNodes[229071] = { -- Sea Scorpion School
			id = 229071,
			name = TFN("Sea Scorpion School"),
			expansion = 5,
			zones = {
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[949] = GetMapNameByID(949), -- Gorgrond
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[111665] = { -- Sea Scorpion
					rate = 1,
				},
				[111658] = { -- Small Sea Scorpion
					rate = -1,
				},
				[111672] = { -- Enormous Sea Scorpion
					rate = -1,
				},
			},
		}

		fishingNodes[218650] = { -- Sha-Touched Spinefish
			id = 218650,
			name = TFN("Sha-Touched Spinefish"),
			expansion = 4,
			zones = {
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
			},
			contains = {
				[83064] = { -- Spinefish
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[94932] = { -- Tiny Red Carp
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[86544] = { -- Spinefish Alpha
					rate = -1,
				},
			},
		}

		fishingNodes[207724] = { -- Shipwreck Debris
			id = 207724,
			name = TFN("Shipwreck Debris"),
			expansion = 3,
			zones = {
				[857] = GetMapNameByID(857), -- Krasarang Wilds
				[709] = GetMapNameByID(709), -- Tol Barad Peninsula
			},
			contains = {
				[52326] = { -- Volatile Water
					rate = -1,
				},
				[52985] = { -- Azshara's Veil
					rate = -1,
				},
				[67597] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[80830] = { -- Rusty Shipwreck Debris
					rate = -1,
				},
				[22739] = { -- Tome of Polymorph: Turtle
					rate = -1,
				},
			},
		}

		fishingNodes[212177] = { -- Spinefish School
			id = 212177,
			name = TFN("Spinefish School"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[810] = GetMapNameByID(810), -- Townlong Steppes
			},
			contains = {
				[83064] = { -- Spinefish
					rate = 1,
				},
				[94932] = { -- Tiny Red Carp
					rate = -1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86544] = { -- Spinefish Alpha
					rate = -1,
				},
			},
		}

		fishingNodes[182953] = { -- Sporefish School
			id = 182953,
			name = TFN("Sporefish School"),
			expansion = 1,
			zones = {
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[27511] = { -- Inscribed Scrollcase
					rate = -1,
				},
				[27513] = { -- Curious Crate
					rate = -1,
				},
				[22578] = { -- Mote of Water
					rate = -1,
				},
				[27442] = { -- Goldenscale Vendorfish
					rate = -1,
				},
				[27429] = { -- Zangarian Sporefish
					rate = -1,
				},
			},
		}

		fishingNodes[182952] = { -- Steam Pump Flotsam
			id = 182952,
			name = TFN("Steam Pump Flotsam"),
			expansion = 1,
			zones = {
				[467] = GetMapNameByID(467), -- Zangarmarsh
			},
			contains = {
				[34476] = { -- Broken Engine Part
					rate = -1,
				},
				[34469] = { -- Strange Engine Part
					rate = -1,
				},
				[27513] = { -- Curious Crate
					rate = -1,
				},
				[27481] = { -- Heavy Supply Crate
					rate = -1,
				},
				[22578] = { -- Mote of Water
					rate = -1,
				},
				[27511] = { -- Inscribed Scrollcase
					rate = -1,
				},
				[27442] = { -- Goldenscale Vendorfish
					rate = -1,
				},
				[27443] = { -- Steam Pump Debris
					rate = -1,
				},
			},
		}

		fishingNodes[180712] = { -- Stonescale Eel Swarm
			id = 180712,
			name = TFN("Stonescale Eel Swarm"),
			expansion = 0,
			zones = {
				[19] = GetMapNameByID(19), -- Blasted Lands
				[38] = GetMapNameByID(38), -- Swamp of Sorrows
				[161] = GetMapNameByID(161), -- Tanaris
				[61] = GetMapNameByID(61), -- Thousand Needles
			},
			contains = {
				[13422] = { -- Stonescale Eel
					rate = 1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[21228] = { -- Mithril Bound Trunk
					rate = -1,
				},
				[3820] = { -- Stranglekelp
					rate = -1,
				},
			},
		}

		fishingNodes[184845] = { -- Strange Pool
			id = 184845,
			name = TFN("Strange Pool"),
			expansion = 1,
			zones = {
				[780] = GetMapNameByID(780), -- Coilfang Reservoir: Serpentshrine Cavern
			},
			contains = {
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[21150] = { -- Iron Bound Trunk
					rate = -1,
				},
				[21153] = { -- Raw Greater Sagefish
					rate = -1,
				},
			},
		}

		fishingNodes[218631] = { -- Swarm of Panicked Paddlefish
			id = 218631,
			name = TFN("Swarm of Panicked Paddlefish"),
			expansion = 4,
			zones = {
				[857] = GetMapNameByID(857), -- Krasarang Wilds
			},
			contains = {
				[74865] = { -- Krasarang Paddlefish
					rate = -1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94934] = { -- Tiny Green Carp
					rate = -1,
				},
			},
		}

		fishingNodes[218577] = { -- Tangled Mantis Shrimp Cluster
			id = 218577,
			name = TFN("Tangled Mantis Shrimp Cluster"),
			expansion = 4,
			zones = {
				[858] = GetMapNameByID(858), -- Dread Wastes
			},
			contains = {
				[86545] = { -- Mimic Octopus
					rate = -1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[74857] = { -- Giant Mantis Shrimp
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
			},
		}

		fishingNodes[212175] = { -- Tiger Gourami School
			id = 212175,
			name = TFN("Tiger Gourami School"),
			expansion = 4,
			zones = {
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
				[873] = GetMapNameByID(873), -- The Veiled Stair
			},
			contains = {
				[74861] = { -- Tiger Gourami
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
			},
		}

		fishingNodes[218635] = { -- Tiger Gourami Slush
			id = 218635,
			name = TFN("Tiger Gourami Slush"),
			expansion = 4,
			zones = {
				[809] = GetMapNameByID(809), -- Kun-Lai Summit
			},
			contains = {
				[74861] = { -- Tiger Gourami
					rate = 1,
				},
				[88496] = { -- Sealed Crate
					rate = -1,
				},
				[97981] = { -- Impeccably Sharp Tooth
					rate = -1,
				},
				[46109] = { -- Sea Turtle
					rate = -1,
				},
				[86542] = { -- Flying Tiger Gourami
					rate = -1,
				},
				[94935] = { -- Tiny White Carp
					rate = -1,
				},
			},
		}

		fishingNodes[180685] = { -- Waterlogged Wreckage
			id = 180685,
			name = TFN("Waterlogged Wreckage"),
			expansion = 0,
			zones = {
				[101] = GetMapNameByID(101), -- Desolace
				[141] = GetMapNameByID(141), -- Dustwallow Marsh
				[121] = GetMapNameByID(121), -- Feralas
				[22] = GetMapNameByID(22), -- Western Plaguelands
			},
			contains = {
				[3820] = { -- Stranglekelp
					rate = -1,
				},
				[21151] = { -- Rumsey Rum Black Label
					rate = -1,
				},
				[21150] = { -- Iron Bound Trunk
					rate = -1,
				},
				[21164] = { -- Bloated Rockscale Cod
					rate = -1,
				},
			},
		}

	end
	
	-- LOGGING
	local loggingNodes = {}

	do -- node data
		loggingNodes[233604] = { -- Small Timber
			id = 233604,
			name = TLN("Small Timber"),
			expansion = 5,
			zones = {
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[949] = GetMapNameByID(949), -- Gorgrond
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[114781] = { -- Timber
					rate = 1,
					minAmount = 3,
					maxAmount = 7,
				},
			},
		}

		loggingNodes[233634] = { -- Timber
			id = 233634,
			name = TLN("Timber"),
			expansion = 5,
			zones = {
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[949] = GetMapNameByID(949), -- Gorgrond
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[114781] = { -- Timber
					rate = 1,
					minAmount = 8,
					maxAmount = 12,
				},
			},
		}

		loggingNodes[233635] = { -- Large Timber
			id = 233635,
			name = TLN("Large Timber"),
			expansion = 5,
			zones = {
				[941] = GetMapNameByID(941), -- Frostfire Ridge
				[949] = GetMapNameByID(949), -- Gorgrond
				[950] = GetMapNameByID(950), -- Nagrand
				[947] = GetMapNameByID(947), -- Shadowmoon Valley
				[948] = GetMapNameByID(948), -- Spires of Arak
				[946] = GetMapNameByID(946), -- Talador
			},
			contains = {
				[114781] = { -- Timber
					rate = 1,
					minAmount = 18,
					maxAmount = 22,
				},
			},
		}
	end
	
	-- setup DBs
	SetupDBs(NTI.PROFESSION_MINING,    miningNodes)
	SetupDBs(NTI.PROFESSION_HERBALISM, herbalismNodes)
	SetupDBs(NTI.PROFESSION_FISHING,   fishingNodes)
	SetupDBs(NTI.PROFESSION_LOGGING,   loggingNodes)
end
